@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "misc.h"
 #include <gdk/gdkkeysyms.h>
@@ -31,7 +31,7 @@ message_box_clicked (GtkWidget *widget,
                      GtkWidget *dialog)
 {
   *(gint *)g_object_get_data (G_OBJECT (dialog), "user_data")
-                    = (gint)g_object_get_data (G_OBJECT (widget), "user_data");
+        = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "user_data"));
   gtk_widget_destroy (dialog);
 }
 
@@ -82,7 +82,7 @@ misc_message_box (const gchar *title,
       button = gtk_button_new_with_mnemonic (name);
       g_signal_connect (G_OBJECT (button), "clicked",
                                     G_CALLBACK (message_box_clicked), dialog);
-      g_object_set_data (G_OBJECT (button), "user_data", (gpointer)i);
+      g_object_set_data (G_OBJECT (button), "user_data", GINT_TO_POINTER (i));
       GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
       if (i == def)
         focus = button;
@@ -660,7 +660,7 @@ misc_str_to_float (gint        *value,
      flag,TRUE:符号あり,FALSE:符号なし
       RET,文字列                                                            */
 gchar *
-misc_str_from_array (const gpointer *array,
+misc_str_from_array (gconstpointer  *array,
                      const gsize     size,
                      const gint      bits,
                      const gint      radix,
