@@ -14,13 +14,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "fileio.h"
 #include "misc.h"
 #include "profile.h"
 
 
+#define PROFILE_DATA_TYPE_UNKNOW 0
+#define PROFILE_DATA_TYPE_SPACE 1
+#define PROFILE_DATA_TYPE_COMMENT 2
+#define PROFILE_DATA_TYPE_SECTION 3
+#define PROFILE_DATA_TYPE_KEY 4
+
+
 typedef struct _ProfileList
 {
   gchar *data, *section, *key, *value;
@@ -37,109 +44,127 @@ struct _Profile
 
 /******************************************************************************
 *                                                                             *
-* ja:初期化ファイル関数群                                                     *
+* ja:初期化ファイル関数群(内部)                                               *
 *                                                                             *
 ******************************************************************************/
-/*  ja:ファイルをリスト化する
-    file,ファイル名
+/*  ja:テキストをリスト化する
+    text,テキスト
      RET,リスト                                                             */
 static ProfileList *
-profile_list (const gchar *file)
+profile_list_from_text (const gchar *text)
 {
-  gchar buf[256], *data = NULL, *tmp, *section = NULL;
-  gint n;
-  FileIO *fio;
+  gchar **buf, *section = NULL;
+  gint i;
   ProfileList *p = NULL, *q, *r;
 
-  fio = fileio_open (file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
-                                                    FILEIO_MODE_OPEN_EXISTING);
-  if (!fio)
-      return NULL;
-  while (fileio_gets (buf, 256, fio))
+  buf = g_strsplit (text, "\n", 0);
+  for (i = 0; buf[i]; i++)
     {
-      if (!data)
-        {
-          data = g_strdup (buf);
-        }
-      else
-        {
-          tmp = data;
-          data = g_strconcat (tmp, buf, NULL);
-          g_free (tmp);
-        }
-      n = data ? g_strlen (data) : 0;
-      if (n > 0 && data[n - 1] == '\n')
-        {
-          data[n - 1] = '\0';
-          q = g_malloc0 (sizeof (ProfileList));
-          q->data = data;
-          q->prev = p;
-          if (p)
-            p->next = q;
-          p = q;
-          data = NULL;
-        }
-    }
-  if (!fileio_close (fio) || !p)
-    {
-      while (p)
-        {
-          q = p->prev;
-          g_free (p->data);
-          g_free (p);
-          p = q;
-        }
-      return NULL;
+      q = g_malloc0 (sizeof (ProfileList));
+      q->data = g_strdup (buf[i]);
+      q->prev = p;
+      if (p)
+        p->next = q;
+      p = q;
     }
+  g_strfreev (buf);
   while (p->prev)
     p = p->prev;
   r = p;
   /* ja:リストを整理する */
-    while (p)
-      {
-        data = p->data ? g_strstrip (g_strdup (p->data)) : NULL;
-        n = data ? g_strlen (data) : 0;
-        if (n <= 0)
-          {
-            p->type = PROFILE_DATA_TYPE_SPACE;
-          }
-        else if (data[0] == '#' || data[0] == ';')
-          {
-            p->type = PROFILE_DATA_TYPE_COMMENT;
-          }
-        else if (data[0] == '[' && data[n - 1] == ']')
-          {
-            p->type = PROFILE_DATA_TYPE_SECTION;
-            g_free (section);
-            section = g_strdup (data + 1);
-            section[n - 2] = '\0';
-            for (q = p->prev; q && q->type == PROFILE_DATA_TYPE_SPACE;
+  while (p)
+    {
+      gsize n;
+      gchar *data;
+
+      data = p->data ? g_strstrip (g_strdup (p->data)) : NULL;
+      n = data ? g_strlen (data) : 0;
+      if (n <= 0)
+        {
+          p->type = PROFILE_DATA_TYPE_SPACE;
+        }
+      else if (data[0] == '#' || data[0] == ';')
+        {
+          p->type = PROFILE_DATA_TYPE_COMMENT;
+        }
+      else if (data[0] == '[' && data[n - 1] == ']')
+        {
+          p->type = PROFILE_DATA_TYPE_SECTION;
+          g_free (section);
+          section = g_strdup (data + 1);
+          section[n - 2] = '\0';
+          for (q = p->prev; q && q->type == PROFILE_DATA_TYPE_SPACE;
                                                                 q = q->prev)
-              {
-                g_free (q->section);
-                q->section = NULL;
-              }
-          }
-        else if (g_strchr (data, '='))
-          {
-            p->type = PROFILE_DATA_TYPE_KEY;
-            p->key = g_strdup (data);
-            *g_strchr (p->key, '=') = '\0';
-            p->value = g_strchr (p->data, '=') + 1;
-          }
-        else
-          {
-            p->type = PROFILE_DATA_TYPE_UNKNOW;
-          }
-        p->section = g_strdup (section);
-        g_free (data);
-        p = p->next;
-      }
-    g_free (section);
-    return r;
+            {
+              g_free (q->section);
+              q->section = NULL;
+            }
+        }
+      else if (g_strchr (data, '='))
+        {
+          p->type = PROFILE_DATA_TYPE_KEY;
+          p->key = g_strdup (data);
+          *g_strchr (p->key, '=') = '\0';
+          p->value = g_strchr (p->data, '=') + 1;
+        }
+      else
+        {
+          p->type = PROFILE_DATA_TYPE_UNKNOW;
+        }
+      p->section = g_strdup (section);
+      g_free (data);
+      p = p->next;
+    }
+  g_free (section);
+  return r;
 }
 
 
+/*  ja:ファイルをリスト化する
+    file,ファイル名
+     RET,リスト                                                             */
+static ProfileList *
+profile_list_from_file (const gchar *file)
+{
+  gchar *data;
+  gsize length;
+  ProfileList *p = NULL;
+
+  data = fileio_load (file, &length);
+  if (data)
+    {
+      data = g_realloc (data, length + 1);
+      data[length] = '\0';
+      p = profile_list_from_text (data);
+      g_free (data);
+    }
+  return p;
+}
+
+
+/*  ja:初期化ファイルを解放する
+    profile_list,リスト                                                     */
+static void
+profile_list_free (ProfileList *profile_list)
+{
+  ProfileList *p, *q;
+
+  for (p = profile_list; p; p = q)
+    {
+      q = p->next;
+      g_free (p->data);
+      g_free (p->section);
+      g_free (p->key);
+      g_free (p);
+    }
+}
+
+
+/******************************************************************************
+*                                                                             *
+* ja:初期化ファイル関数群(独自)                                               *
+*                                                                             *
+******************************************************************************/
 /*  ja:初期化ファイルを開く
     file,ファイル名
      RET,プロファイル                                                       */
@@ -190,7 +215,7 @@ profile_open (const gchar *file)
           sys = g_strconcat (dir, G_DIR_SEPARATOR_S, tmp, ".conf", NULL);
           g_free (dir);
 #endif /* not SYSCONFDIR */
-          profile->sublist = profile_list (sys);
+          profile->sublist = profile_list_from_file (sys);
           g_free (sys);
         }
       g_free (tmp);
@@ -202,7 +227,7 @@ profile_open (const gchar *file)
       : g_strconcat (g_get_home_dir (), G_DIR_SEPARATOR_S, ".", base, NULL);
 #endif /* not G_OS_WIN32 */
   g_free (base);
-  profile->list = profile_list (profile->file);
+  profile->list = profile_list_from_file (profile->file);
   return profile;
 }
 
@@ -214,7 +239,7 @@ gboolean
 profile_close (Profile *profile)
 {
   FileIO *fio;
-  ProfileList *p, *q;
+  ProfileList *p;
 
   if (!profile)
     return TRUE;
@@ -231,23 +256,9 @@ profile_close (Profile *profile)
         }
       fileio_close (fio);
     }
+  profile_list_free (profile->list);
+  profile_list_free (profile->sublist);
   g_free (profile->file);
-  for (p = profile->list; p; p = q)
-    {
-      q = p->next;
-      g_free (p->data);
-      g_free (p->section);
-      g_free (p->key);
-      g_free (p);
-    }
-  for (p = profile->sublist; p; p = q)
-    {
-      q = p->next;
-      g_free (p->data);
-      g_free (p->section);
-      g_free (p->key);
-      g_free (p);
-    }
   g_free (profile);
   return TRUE;
 }
@@ -266,7 +277,7 @@ profile_get_string (Profile     *profile,
   ProfileList *p;
 
   if (!profile || !section || !key)
-    return FALSE;
+    return NULL;
   for (p = profile->list; p; p = p->next)
     if (p->type == PROFILE_DATA_TYPE_KEY && g_strcmp (p->section, section) == 0
                                                 && g_strcmp (p->key, key) == 0)
@@ -285,14 +296,14 @@ profile_get_string (Profile     *profile,
         key,キー
        type,タイプ
         RET,バイト数,0:エラー                                               */
-gint
+gsize
 profile_get_size (Profile     *profile,
                   const gchar *section,
                   const gchar *key,
                   const guint  type)
 {
   guint8 *array;
-  gint n;
+  gsize n;
   ProfileList *p;
 
   if (!profile || !section || !key)
@@ -348,7 +359,7 @@ profile_get_value (Profile     *profile,
                    const gchar *section,
                    const gchar *key,
                    gpointer     value,
-                   const gint   size,
+                   const gsize  size,
                    const guint  type)
 {
   guint8 *array;
@@ -390,7 +401,7 @@ profile_get_value (Profile     *profile,
         if (size < sizeof (gint))
           return FALSE;
         misc_str_to_val ((gint *)value, p->value, 10, TRUE);
-          break;
+        break;
       case PROFILE_VALUE_TYPE_STRING:
         if (size < g_strlen (p->value) + 1)
           return FALSE;
@@ -426,7 +437,7 @@ profile_set_value (Profile       *profile,
                    const gchar   *section,
                    const gchar   *key,
                    gconstpointer  value,
-                   const gint     size,
+                   const gsize    size,
                    const guint    type)
 {
   gchar *data;
@@ -457,7 +468,7 @@ profile_set_value (Profile       *profile,
               q->next = p;
               q = p;
             }
-            /* ja:セクションもキーもないときにはセクションを作る */
+          /* ja:セクションもキーもないときにはセクションを作る */
           p = g_malloc (sizeof (ProfileList));
           p->type = PROFILE_DATA_TYPE_SECTION;
           p->data = g_strdup_printf ("[%s]", section);
@@ -498,7 +509,7 @@ profile_set_value (Profile       *profile,
         g_free (p->data);
         p->data = g_strdup_printf ("%s=%s",
                             p->key, *((gboolean *)value) ? "true" : "false");
-          break;
+        break;
       case PROFILE_VALUE_TYPE_INT:
         g_free (p->data);
         p->data = g_strdup_printf ("%s=%d", p->key, *((gint *)value));
@@ -520,6 +531,7 @@ profile_set_value (Profile       *profile,
       default:
         return FALSE;
     }
+  p->type = type;
   p->value = g_strchr (p->data, '=') + 1;
   profile->edit = TRUE;
   return TRUE;
@@ -645,3 +657,355 @@ profile_enum_key (Profile     *profile,
       glist = g_list_insert_sorted (glist, p->key, (GCompareFunc)strcmp);
   return glist;
 }
+
+
+/******************************************************************************
+*                                                                             *
+* ja:初期化ファイル関数群(互換)                                               *
+*                                                                             *
+******************************************************************************/
+#if ! GLIB_CHECK_VERSION(2,6,0)
+GKeyFile *
+g_key_file_new (void)
+{
+  return g_malloc0 (sizeof (Profile));
+}
+
+
+void
+g_key_file_free (GKeyFile *key_file)
+{
+  if (key_file)
+    {
+      profile_list_free (key_file->list);
+      profile_list_free (key_file->sublist);
+      g_free (key_file->file);
+      g_free (key_file);
+    }
+}
+
+
+gboolean
+g_key_file_load_from_file (GKeyFile       *key_file,
+                           const gchar    *file,
+                           GKeyFileFlags   flags,
+                           GError        **error)
+{
+  gboolean result = FALSE;
+
+  if (error)
+    *error = NULL;
+  if (key_file)
+    {
+      gchar *data;
+      gsize length;
+
+      data = fileio_load (file, &length);
+      if (data)
+        result = g_key_file_load_from_data (key_file, data, length,
+                                                                flags, error);
+      g_free (data);
+    }
+  return result;
+}
+
+
+gboolean
+g_key_file_load_from_data (GKeyFile       *key_file,
+                           const gchar    *data,
+                           gsize           length,
+                           GKeyFileFlags   flags,
+                           GError        **error)
+{
+  if (error)
+    *error = NULL;
+  if (!key_file)
+    return FALSE;
+  /* ja:リスト解放 */
+  profile_list_free (key_file->list);
+  profile_list_free (key_file->sublist);
+  key_file->list = key_file->sublist = NULL;
+  if (data && length != 0)
+    {
+      gchar *text;
+
+      text = length == (gsize)-1 ? g_strdup (data) : g_strndup (data, length);
+      key_file->list = profile_list_from_text (text);
+      g_free (text);
+    }
+  return TRUE;
+}
+
+
+gchar *
+g_key_file_to_data (GKeyFile  *key_file,
+                    gsize     *length,
+                    GError   **error)
+{
+  gchar *data = NULL;
+
+  if (error)
+    *error = NULL;
+  if (key_file)
+    {
+      ProfileList *p;
+
+      for (p = key_file->list; p; p = p->next)
+        {
+          gchar *tmp;
+
+          tmp = data;
+          data = data ? p->data ? g_strconcat (data, p->data, "\n", NULL)
+                                : g_strconcat (data,          "\n", NULL)
+                                : g_strconcat (      p->data, "\n", NULL);
+          g_free (tmp);
+        }
+    }
+  if (data && length)
+    *length = g_strlen (data);
+  return data;
+}
+
+
+gchar **
+g_key_file_get_groups (GKeyFile *key_file,
+                       gsize    *length)
+{
+  gchar **str = NULL;
+  GList *glist;
+
+  glist = profile_enum_section (key_file);
+  if (glist)
+    {
+      guint i, leng;
+
+      leng = g_list_length (glist);
+      str = g_malloc (leng * sizeof (gchar *));
+      for (i = 0; i < leng; i++)
+        str[i] = g_strdup (g_list_nth_data (glist, i));
+      str[i] = NULL;
+      g_list_free (glist);
+      if (length)
+        *length = leng;
+    }
+  return str;
+}
+
+
+gchar **
+g_key_file_get_keys (GKeyFile     *key_file,
+                     const gchar  *group_name,
+                     gsize        *length,
+                     GError      **error)
+{
+  gchar **str = NULL;
+  GList *glist;
+
+  if (error)
+    *error = NULL;
+  glist = profile_enum_key (key_file, group_name);
+  if (glist)
+    {
+      guint i, leng;
+
+      leng = g_list_length (glist);
+      str = g_malloc (leng * sizeof (gchar *));
+      for (i = 0; i < leng; i++)
+        str[i] = g_strdup (g_list_nth_data (glist, i));
+      str[i] = NULL;
+      g_list_free (glist);
+      if (length)
+        *length = leng;
+    }
+  return str;
+}
+
+
+gboolean
+g_key_file_has_group (GKeyFile    *key_file,
+                      const gchar *group_name)
+{
+  if (key_file && group_name)
+    {
+      ProfileList *p;
+
+      for (p = key_file->list; p; p = p->next)
+        if (p->section && g_strcmp (p->section, group_name) == 0)
+          return TRUE;
+      for (p = key_file->sublist; p; p = p->next)
+        if (p->section && g_strcmp (p->section, group_name) == 0)
+          return TRUE;
+    }
+  return FALSE;
+}
+
+
+gboolean
+g_key_file_has_key (GKeyFile     *key_file,
+                    const gchar  *group_name,
+                    const gchar  *key,
+                    GError      **error)
+{
+  if (error)
+    *error = NULL;
+  if (key_file && group_name && key)
+    {
+      ProfileList *p;
+
+      for (p = key_file->list; p; p = p->next)
+        if (p->section && p->key && g_strcmp (p->section, group_name) == 0
+                                                && g_strcmp (p->key, key) == 0)
+          return TRUE;
+      for (p = key_file->sublist; p; p = p->next)
+        if (p->section && p->key && g_strcmp (p->section, group_name) == 0
+                                                && g_strcmp (p->key, key) == 0)
+          return TRUE;
+    }
+  return FALSE;
+}
+
+
+gchar *
+g_key_file_get_string (GKeyFile     *key_file,
+                       const gchar  *group_name,
+                       const gchar  *key,
+                       GError      **error)
+{
+  if (error)
+    *error = NULL;
+  return profile_get_string (key_file, group_name, key);
+}
+
+
+gboolean
+g_key_file_get_boolean (GKeyFile     *key_file,
+                        const gchar  *group_name,
+                        const gchar  *key,
+                        GError      **error)
+{
+  gboolean value;
+
+  if (error)
+    *error = NULL;
+  if (!profile_get_value (key_file, group_name, key,
+                (gpointer)&value, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL))
+    value = FALSE;
+  return value;
+}
+
+
+gint
+g_key_file_get_integer (GKeyFile     *key_file,
+                        const gchar  *group_name,
+                        const gchar  *key,
+                        GError      **error)
+{
+  gint value;
+
+  if (error)
+    *error = NULL;
+  if (!profile_get_value (key_file, group_name, key,
+                    (gpointer)&value, sizeof (gint), PROFILE_VALUE_TYPE_INT))
+    value = 0;
+  return value;
+}
+
+
+gint *
+g_key_file_get_integer_list (GKeyFile     *key_file,
+                             const gchar  *group_name,
+                             const gchar  *key,
+                             gsize        *length,
+                             GError      **error)
+{
+  gsize leng;
+  gint *value = NULL;
+
+  if (error)
+    *error = NULL;
+  leng = profile_get_size (key_file, group_name, key,
+                                                    PROFILE_VALUE_TYPE_ARRAY);
+  if (leng > 0)
+    {
+      value = g_malloc (leng);
+      if (!profile_get_value (key_file, group_name, key,
+                            (gpointer)value, leng, PROFILE_VALUE_TYPE_ARRAY))
+        {
+          g_free (value);
+          value = NULL;
+          leng = 0;
+        }
+    }
+  if (length)
+    *length = leng / sizeof (gint);
+  return value;
+}
+
+
+void
+g_key_file_set_string (GKeyFile    *key_file,
+                       const gchar *group_name,
+                       const gchar *key,
+                       const gchar *string)
+{
+  profile_set_value (key_file, group_name, key, (gpointer)string,
+        (g_strlen (string) + 1) * sizeof (gchar), PROFILE_VALUE_TYPE_STRING);
+}
+
+
+void
+g_key_file_set_boolean (GKeyFile    *key_file,
+                        const gchar *group_name,
+                        const gchar *key,
+                        gboolean     value)
+{
+  profile_set_value (key_file, group_name, key,
+                (gpointer)&value, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
+}
+
+
+void
+g_key_file_set_integer (GKeyFile    *key_file,
+                        const gchar *group_name,
+                        const gchar *key,
+                        gint         value)
+{
+  profile_set_value (key_file, group_name, key,
+                    (gpointer)&value, sizeof (gint), PROFILE_VALUE_TYPE_INT);
+}
+
+
+void
+g_key_file_set_integer_list (GKeyFile    *key_file,
+                             const gchar *group_name,
+                             const gchar *key,
+                             gint         list[],
+                             gsize        length)
+{
+  profile_set_value (key_file, group_name, key,
+            (gpointer)list, length * sizeof (gint), PROFILE_VALUE_TYPE_ARRAY);
+}
+
+
+void
+g_key_file_remove_group (GKeyFile     *key_file,
+                         const gchar  *group_name,
+                         GError      **error)
+{
+  if (error)
+    *error = NULL;
+  profile_delete_section (key_file, group_name);
+}
+
+
+void
+g_key_file_remove_key (GKeyFile     *key_file,
+                       const gchar  *group_name,
+                       const gchar  *key,
+                       GError      **error)
+{
+  if (error)
+    *error = NULL;
+  profile_delete_key (key_file, group_name, key);
+}
+#endif /* not GLIB_CHECK_VERSION(2,6,0) */
