@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "abort.h"
 #include "add.h"
@@ -31,7 +31,6 @@
 #include "root.h"
 #include "version.h"
 #include "misc/fileio.h"
-#include "misc/misc.h"
 #include "orz/orzcharlist.h"
 #include "orz/orzcharset.h"
 #include "orz/orzconf.h"
@@ -55,21 +54,6 @@ command_new (GtkWidget *widget,
 }
 
 
-#if ! GTK_CHECK_VERSION(2,4,0)
-static void
-command_open_ok (GtkWidget *widget,
-                 GtkWidget *dialog)
-{
-  gchar **file;
-
-  file = g_object_get_data (G_OBJECT (dialog), "user_data");
-  *file
-    = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog)));
-  gtk_widget_destroy (dialog);
-}
-#endif /* not GTK_CHECK_VERSION(2,4,0) */
-
-
 void
 command_open (GtkWidget *widget,
               gpointer   user_data)
@@ -87,17 +71,8 @@ command_open (GtkWidget *widget,
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), open_path);
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
     file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-  gtk_widget_destroy (dialog);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
   dialog = gtk_file_selection_new (_("Open"));
-  g_signal_connect (G_OBJECT (dialog), "destroy",
-                                            G_CALLBACK (gtk_main_quit), NULL);
-  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (dialog)->ok_button),
-                            "clicked", G_CALLBACK (command_open_ok), dialog);
-  g_signal_connect_swapped
-                        (G_OBJECT (GTK_FILE_SELECTION (dialog)->cancel_button),
-            "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
-  g_object_set_data (G_OBJECT (dialog), "user_data", &file);
   if (open_path)
     {
       path = g_strconcat (open_path, G_SEARCHPATH_SEPARATOR_S, NULL);
@@ -105,10 +80,11 @@ command_open (GtkWidget *widget,
       g_free (path);
     }
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (dialog));
-  gtk_widget_show (dialog);
-  gtk_grab_add (dialog);
-  gtk_main ();
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
+    file = g_strdup (gtk_file_selection_get_filename
+                                                (GTK_FILE_SELECTION (dialog)));
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_widget_destroy (dialog);
   if (file)
     {
       g_free (open_path);
@@ -171,16 +147,23 @@ command_save (GtkWidget *widget,
     }
   else
     {
-      gchar *text, *utf8str;
+      gboolean result = TRUE;
 
-      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-      text = g_strdup_printf (_("%s\nThis file is existed.\n\nOver write?"),
-                                                                    utf8str);
-      g_free (utf8str);
-      if ((!fileio_isfile (file)
-            || misc_message_box ("Melody maid", text, 1,
-                                            _("_Yes"), _("_No"), NULL) == 0)
-                        &&  (mmaid->filekind == FILE_KIND_DB
+      if (fileio_isfile (file))
+        {
+          gchar *utf8str;
+          GtkWidget *dialog;
+
+          utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+          dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                    GTK_MESSAGE_INFO, GTK_BUTTONS_YES_NO,
+                    _("%s\nThis file is existed.\n\nOver write?"), utf8str);
+          g_free (utf8str);
+          result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
+          gtk_widget_destroy (dialog);
+        }
+      if (result &&  (mmaid->filekind == FILE_KIND_DB
                         ? file_save_db (file, mmaid, root, mmaid->charset)
                         : file_save_list (file, mmaid, root, mmaid->charset,
                                                             mmaid->playlist))
@@ -191,98 +174,90 @@ command_save (GtkWidget *widget,
 }
 
 
-typedef struct _CommandSaveOk CommandSaveOk;
-struct _CommandSaveOk
-{
-  gchar *file, *root, *charset;
-  guint filekind;
-  gint playlist;
-  GtkWidget *dialog, *hbox, *combo0, *combo1;
-  GtkWidget *radio0, *radio1, *radio2, *radio3, *radio4;
-};
-
-
 /* ja:ラジオボタンが変更された */
 static void
-command_save_toggled_radio0 (GtkWidget     *widget,
-                             CommandSaveOk *cmd_sv)
+command_save_toggled_radio0 (GtkWidget *widget,
+                             GtkWidget *hbox)
 {
-  gtk_widget_set_sensitive (cmd_sv->hbox, FALSE);
+  gtk_widget_set_sensitive (hbox, FALSE);
 }
 
 
 /* ja:ラジオボタンが変更された */
 static void
-command_save_toggled_radio1 (GtkWidget     *widget,
-                             CommandSaveOk *cmd_sv)
+command_save_toggled_radio1 (GtkWidget *widget,
+                             GtkWidget *hbox)
 {
-  gtk_widget_set_sensitive (cmd_sv->hbox, TRUE);
-}
-
-
-#if ! GTK_CHECK_VERSION(2,4,0)
-static void
-command_save_ok (GtkWidget     *widget,
-                 CommandSaveOk *cmd_sv)
-{
-  cmd_sv->file    = g_strdup (gtk_file_selection_get_filename
-                                        (GTK_FILE_SELECTION (cmd_sv->dialog)));
-  cmd_sv->root    = g_strdup (gtk_entry_get_text
-                            (GTK_ENTRY (GTK_COMBO (cmd_sv->combo)->entry)));
-  cmd_sv->charset = orz_charset_get_charset (ORZ_CHARSET (cmd_sv->combo1));
-  cmd_sv->filekind = gtk_toggle_button_get_active
-        (GTK_TOGGLE_BUTTON (cmd_sv->radio0)) ? FILE_KIND_DB : FILE_KIND_LIST;
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cmd_sv->radio3)))
-    cmd_sv->playlist = 1;
-  else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cmd_sv->radio4)))
-    cmd_sv->playlist = 2;
-  else
-    cmd_sv->playlist = 0;
-  gtk_widget_destroy (cmd_sv->dialog);
+  gtk_widget_set_sensitive (hbox, TRUE);
 }
 
 
 /* ja:コンボボックスが変更された */
 static void
-command_save_changed (GtkWidget     *widget,
-                      CommandSaveOk *cmd_sv)
+command_save_changed (GtkWidget *widget,
+                      GtkWidget *dialog)
 {
-  gchar *file, *name;
+  gchar *root, *utf8str;
+
+#if GTK_CHECK_VERSION(2,4,0)
+# if GTK_CHECK_VERSION(2,6,0)
+  utf8str = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
+# else /* not GTK_CHECK_VERSION(2,6,0) */
+  GtkTreeIter iter;
+
+  if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter))
+        gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (widget)),
+                                                    &iter, 0, &utf8str, -1);
+# endif /* not GTK_CHECK_VERSION(2,6,0) */
+  root = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), root);
+  g_free (root);
+  g_free (utf8str);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  gchar *file, *name, *root;
 
   name = g_path_get_basename (gtk_file_selection_get_filename
-                                        (GTK_FILE_SELECTION (cmd_sv->dialog)));
-  file = g_strconcat (gtk_entry_get_text (GTK_ENTRY (widget)), name, NULL);
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (cmd_sv->dialog), file);
+                                                (GTK_FILE_SELECTION (dialog)));
+  root = g_filename_from_utf8
+            (gtk_entry_get_text (GTK_ENTRY (widget)), -1, NULL, NULL, NULL);
+  file = g_strconcat (root, name, NULL);
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), file);
   g_free (file);
   g_free (name);
+  g_free (root);
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
 }
 
 
 /* ja:コンボボックスが変更された */
 static void
-command_save_edited (GtkWidget     *widget,
-                     CommandSaveOk *cmd_sv)
+command_save_edited (GtkWidget *widget,
+                     GtkWidget *dialog)
 {
-  gtk_widget_set_sensitive (GTK_FILE_SELECTION (cmd_sv->dialog)->ok_button,
+#if GTK_CHECK_VERSION(2,4,0)
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT,
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK,
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
                                 orz_charset_is_valid (ORZ_CHARSET (widget)));
 }
-#endif /* not GTK_CHECK_VERSION(2,4,0) */
 
 
 void
 command_saveas (GtkWidget *widget,
                 gpointer   user_data)
 {
-  gchar *name, *path, *root = NULL, *tmp, **ary;
-  gint i, page_num;
+  gchar *file, *root = NULL, *charset, *name, *path, *tmp, **ary;
+  gint playlist, i, page_num;
+  guint filekind;
   GList *glist = NULL;
-  GtkWidget *label0, *label1, *hbox, *vbox;
-  CommandSaveOk cmd_sv;
+  GtkWidget *dialog, *hbox0, *hbox1, *vbox;
+  GtkWidget *combo0, *combo1, *label0, *label1;
+  GtkWidget *radio0, *radio1, *radio2, *radio3, *radio4;
   MmaidWindow *mmaid;
 
   page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
   mmaid = orz_mdi_get_data (ORZ_MDI (mdi), page_num);
-  g_memset (&cmd_sv, 0 ,sizeof (CommandSaveOk));
   /* ja:共通のルートを求める */
   for (i = mm_list_length (MM_LIST (mmaid->list)) - 1; i >= 0; i--)
     {
@@ -306,8 +281,12 @@ command_saveas (GtkWidget *widget,
   path = g_strrchr (root, G_DIR_SEPARATOR);
   if (!path)
     {
-      misc_message_box ("Melody maid", _("Root Directory Error"), 0,
-                                                            _("OK"), NULL);
+      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                            _("Root Directory Error"));
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
       return;
     }
   *path = '\0';
@@ -316,14 +295,14 @@ command_saveas (GtkWidget *widget,
                 && root[0] == G_DIR_SEPARATOR && root[1] == G_DIR_SEPARATOR)
     {
       path = g_strdup (G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S);
-      ary = g_strsplit (root + 2, G_DIR_SEPARATOR_S, G_MAXINT);
+      ary = g_strsplit (root + 2, G_DIR_SEPARATOR_S, 0);
     }
   else
     {
-      ary = g_strsplit (root, G_DIR_SEPARATOR_S, G_MAXINT);
+      ary = g_strsplit (root, G_DIR_SEPARATOR_S, 0);
     }
 #else /* not G_OS_WIN32 */
-  ary = g_strsplit (root, G_DIR_SEPARATOR_S, G_MAXINT);
+  ary = g_strsplit (root, G_DIR_SEPARATOR_S, 0);
   path = NULL;
 #endif /* not G_OS_WIN32 */
   g_free (root);
@@ -340,227 +319,231 @@ command_saveas (GtkWidget *widget,
       gchar *utf8str;
 
       utf8str = g_filename_to_utf8 (save_path, -1, NULL, NULL, NULL);
-      if (!g_strfilencmp (utf8str, path, g_strlen (utf8str)))
-        root = g_strdup (save_path);
+      if (g_strfilencmp (utf8str, path, g_strlen (utf8str)) == 0)
+        root = g_strconcat (save_path, G_DIR_SEPARATOR_S, NULL);
       g_free (utf8str);
     }
   if (!root)
     root = g_filename_from_utf8 (path, -1, NULL, NULL, NULL);
+  /* ja:ダイアログ */
+#if GTK_CHECK_VERSION(2,4,0)
+  dialog = gtk_file_chooser_dialog_new (_("Save As"),
+                        GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_SAVE,
+                        GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
+                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                        NULL);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+  dialog = gtk_file_selection_new (_("Save As"));
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   /* ja:ラジオボタン */
-  cmd_sv.radio0 = gtk_radio_button_new_with_mnemonic
-                    (NULL, _("_iDB File"));
-  cmd_sv.radio1 = gtk_radio_button_new_with_mnemonic_from_widget
-                    (GTK_RADIO_BUTTON (cmd_sv.radio0), _("_Play List"));
-  cmd_sv.radio2 = gtk_radio_button_new_with_mnemonic
-                    (NULL, _("Rela_tive Path"));
-  cmd_sv.radio3 = gtk_radio_button_new_with_mnemonic_from_widget
-                    (GTK_RADIO_BUTTON (cmd_sv.radio2), _("Ab_solute Path"));
-  cmd_sv.radio4 = gtk_radio_button_new_with_mnemonic_from_widget
-                    (GTK_RADIO_BUTTON (cmd_sv.radio3), _("File Name _Only"));
-  cmd_sv.hbox = gtk_hbox_new (FALSE, 0);
+  radio0 = gtk_radio_button_new_with_mnemonic (NULL, _("_iDB File"));
+  radio1 = gtk_radio_button_new_with_mnemonic_from_widget
+                                (GTK_RADIO_BUTTON (radio0), _("_Play List"));
+  radio2 = gtk_radio_button_new_with_mnemonic (NULL, _("Rela_tive Path"));
+  radio3 = gtk_radio_button_new_with_mnemonic_from_widget
+                            (GTK_RADIO_BUTTON (radio2), _("Ab_solute Path"));
+  radio4 = gtk_radio_button_new_with_mnemonic_from_widget
+                            (GTK_RADIO_BUTTON (radio3), _("File Name _Only"));
+  hbox0 = gtk_hbox_new (FALSE, 0);
   if (mmaid->filekind == FILE_KIND_DB)
     {
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cmd_sv.radio0), TRUE);
-      gtk_widget_set_sensitive (cmd_sv.hbox, FALSE);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio0), TRUE);
+      gtk_widget_set_sensitive (hbox0, FALSE);
     }
   else
     {
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cmd_sv.radio1), TRUE);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio1), TRUE);
     }
   switch (mmaid->playlist)
     {
-      case  1: gtk_toggle_button_set_active
-                            (GTK_TOGGLE_BUTTON (cmd_sv.radio3), TRUE); break;
-      case  2: gtk_toggle_button_set_active
-                            (GTK_TOGGLE_BUTTON (cmd_sv.radio4), TRUE); break;
-      default: gtk_toggle_button_set_active
-                            (GTK_TOGGLE_BUTTON (cmd_sv.radio2), TRUE);
+      case  1: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio3), TRUE);
+               break;
+      case  2: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio4), TRUE);
+               break;
+      default: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio2), TRUE);
     }
-  g_signal_connect (G_OBJECT (cmd_sv.radio0), "toggled",
-                            G_CALLBACK (command_save_toggled_radio0), &cmd_sv);
-  g_signal_connect (G_OBJECT (cmd_sv.radio1), "toggled",
-                            G_CALLBACK (command_save_toggled_radio1), &cmd_sv);
+  g_signal_connect (G_OBJECT (radio0), "toggled",
+                            G_CALLBACK (command_save_toggled_radio0), hbox0);
+  g_signal_connect (G_OBJECT (radio1), "toggled",
+                            G_CALLBACK (command_save_toggled_radio1), hbox0);
   /* ja:コンボボックス */
 #if GTK_CHECK_VERSION(2,4,0)
-  cmd_sv.combo0 = gtk_combo_box_new_text ();
+  combo0 = gtk_combo_box_new_text ();
   for (i = g_list_length (glist) - 1; i >= 0; i--)
     {
-      gtk_combo_box_prepend_text (GTK_COMBO_BOX (cmd_sv.combo0),
+      gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo0),
                                                 g_list_nth_data (glist, i));
       if (g_strfilecmp (g_list_nth_data (glist, i), root) == 0)
-        gtk_combo_box_set_active (GTK_COMBO_BOX (cmd_sv.combo0), 0);
+        gtk_combo_box_set_active (GTK_COMBO_BOX (combo0), 0);
     }
+  g_signal_connect (G_OBJECT (combo0), "changed",
+                                    G_CALLBACK (command_save_changed), dialog);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-  cmd_sv.combo0 = gtk_combo_new ();
-  gtk_combo_set_popdown_strings (GTK_COMBO (cmd_sv.combo0), glist);
-  gtk_combo_set_value_in_list (GTK_COMBO (cmd_sv.combo0), FALSE, FALSE);
-  gtk_combo_set_case_sensitive (GTK_COMBO (cmd_sv.combo0), TRUE);
-  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (cmd_sv.combo0)->entry),
-                                                                        FALSE);
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (cmd_sv.combo0)->entry), root);
+  combo0 = gtk_combo_new ();
+  gtk_combo_set_popdown_strings (GTK_COMBO (combo0), glist);
+  gtk_combo_set_value_in_list (GTK_COMBO (combo0), FALSE, FALSE);
+  gtk_combo_set_case_sensitive (GTK_COMBO (combo0), TRUE);
+  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo0)->entry), FALSE);
+  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo0)->entry), root);
+  g_signal_connect (G_OBJECT (GTK_COMBO (combo0)->entry), "changed",
+                                    G_CALLBACK (command_save_changed), dialog);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
   for (i = g_list_length (glist) - 1; i >= 0; i--)
     g_free (g_list_nth_data (glist, i));
   g_list_free (glist);
   /* ja:文字符号化方式 */
-  cmd_sv.combo1 = orz_charset_new ();
-  orz_charset_add_candidate (ORZ_CHARSET (cmd_sv.combo1), charset_list);
-  orz_charset_set_charset (ORZ_CHARSET (cmd_sv.combo1), mmaid->charset);
+  combo1 = orz_charset_new ();
+  orz_charset_add_candidate (ORZ_CHARSET (combo1), charset_list);
+  orz_charset_set_charset (ORZ_CHARSET (combo1), mmaid->charset);
+  g_signal_connect (G_OBJECT (combo1), "edited",
+                                    G_CALLBACK (command_save_edited), dialog);
   /* ja:ラベル */
   label0 = gtk_label_new_with_mnemonic (_("_Root"));
   label1 = gtk_label_new_with_mnemonic (_("Character _Encoding"));
 #if GTK_CHECK_VERSION(2,4,0)
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label0), cmd_sv.combo0);
+  gtk_label_set_mnemonic_widget (GTK_LABEL(label0), combo0);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label0),
-                                            GTK_COMBO (cmd_sv.combo0)->entry);
+  gtk_label_set_mnemonic_widget (GTK_LABEL(label0), GTK_COMBO (combo0)->entry);
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
-  gtk_label_set_mnemonic_widget (GTK_LABEL(label1), cmd_sv.combo1);
+  gtk_label_set_mnemonic_widget (GTK_LABEL(label1), combo1);
   /* ja:フレームとボックス */
   vbox = gtk_vbox_new (FALSE, SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), cmd_sv.radio0, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), cmd_sv.radio1, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (cmd_sv.hbox),
-                                        gtk_label_new ("("), FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (cmd_sv.hbox), cmd_sv.radio2, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (cmd_sv.hbox), cmd_sv.radio3, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (cmd_sv.hbox), cmd_sv.radio4, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (cmd_sv.hbox),
-                                        gtk_label_new (")"), FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), cmd_sv.hbox, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), label0, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), cmd_sv.combo0, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), cmd_sv.combo1, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+  hbox1 = gtk_hbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox1), radio0, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox1), radio1, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), gtk_label_new ("("), FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), radio2, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), radio3, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), radio4, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), gtk_label_new (")"), FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox1), hbox0, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 0);
+  hbox0 = gtk_hbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), label0, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), combo0, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox0, TRUE, TRUE, 0);
+  hbox0 = gtk_hbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), label1, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox0), combo1, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox0, TRUE, TRUE, 0);
   gtk_widget_show_all (vbox);
   /* ja:ダイアログ */
 #if GTK_CHECK_VERSION(2,4,0)
-  cmd_sv.dialog = gtk_file_chooser_dialog_new (_("Save As"),
-                        GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_SAVE,
-                        GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
-                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                        NULL);
-  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (cmd_sv.dialog), vbox);
+  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dialog), vbox);
   if (root)
-    gtk_file_chooser_set_current_folder
-                                    (GTK_FILE_CHOOSER (cmd_sv.dialog), root);
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), root);
   name = g_path_get_basename (orz_mdi_get_file (ORZ_MDI (mdi), page_num));
   tmp = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
-  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (cmd_sv.dialog), tmp);
+  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), tmp);
+  g_free (root);
   g_free (tmp);
   g_free (name);
-  if (gtk_dialog_run (GTK_DIALOG (cmd_sv.dialog)) == GTK_RESPONSE_ACCEPT)
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
     {
 # if ! GTK_CHECK_VERSION(2,6,0)
       GtkTreeIter iter;
 # endif /* not GTK_CHECK_VERSION(2,6,0) */
 
-      cmd_sv.file = gtk_file_chooser_get_filename
-                                            (GTK_FILE_CHOOSER (cmd_sv.dialog));
+      file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 # if GTK_CHECK_VERSION(2,6,0)
-      cmd_sv.root = gtk_combo_box_get_active_text
-                                            (GTK_COMBO_BOX (cmd_sv.combo0));
+      root = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo0));
 # else /* not GTK_CHECK_VERSION(2,6,0) */
-      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (cmd_sv.combo0), &iter))
-        gtk_tree_model_get
-                    (gtk_combo_box_get_model (GTK_COMBO_BOX (cmd_sv.combo0)),
-                                                &iter, 0, &cmd_sv.root, -1);
+      if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo0), &iter))
+        gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (combo0)),
+                                                        &iter, 0, &root, -1);
 # endif /* not GTK_CHECK_VERSION(2,6,0) */
-      cmd_sv.charset = orz_charset_get_charset (ORZ_CHARSET (cmd_sv.combo1));
-      cmd_sv.filekind = gtk_toggle_button_get_active
-        (GTK_TOGGLE_BUTTON (cmd_sv.radio0)) ? FILE_KIND_DB : FILE_KIND_LIST;
-      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cmd_sv.radio3)))
-        cmd_sv.playlist = 1;
-      else if (gtk_toggle_button_get_active
-                                        (GTK_TOGGLE_BUTTON (cmd_sv.radio4)))
-        cmd_sv.playlist = 2;
+      charset = orz_charset_get_charset (ORZ_CHARSET (combo1));
+      filekind = gtk_toggle_button_get_active
+                (GTK_TOGGLE_BUTTON (radio0)) ? FILE_KIND_DB : FILE_KIND_LIST;
+      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio3)))
+        playlist = 1;
+      else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio4)))
+        playlist = 2;
       else
-        cmd_sv.playlist = 0;
+        playlist = 0;
     }
-  gtk_widget_destroy (cmd_sv.dialog);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
-  cmd_sv.dialog = gtk_file_selection_new (_("Save As"));
-  g_signal_connect (G_OBJECT (cmd_sv.dialog), "destroy",
-                                            G_CALLBACK (gtk_main_quit), NULL);
-  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (cmd_sv.dialog)->ok_button),
-                            "clicked", G_CALLBACK (command_save_ok), &cmd_sv);
-  g_signal_connect_swapped
-                (G_OBJECT (GTK_FILE_SELECTION (cmd_sv.dialog)->cancel_button),
-                                    "clicked", G_CALLBACK (gtk_widget_destroy),
-                                                GTK_OBJECT (cmd_sv.dialog));
-  g_signal_connect_swapped (G_OBJECT (GTK_COMBO (cmd_sv.combo0)->entry),
-                    "activate", G_CALLBACK (gtk_button_clicked),
-                    G_OBJECT (GTK_FILE_SELECTION (cmd_sv.dialog)->ok_button));
-  g_signal_connect_swapped (G_OBJECT (cmd_sv.combo0),
-                    "activate", G_CALLBACK (gtk_button_clicked),
-                    G_OBJECT (GTK_FILE_SELECTION (cmd_sv.dialog)->ok_button));
-  g_signal_connect (G_OBJECT (GTK_COMBO (cmd_sv.combo0)->entry), "changed",
-                                G_CALLBACK (command_save_changed), &cmd_sv);
-  g_signal_connect (G_OBJECT (cmd_sv.combo1), "edited",
-                                G_CALLBACK (command_save_edited), &cmd_sv);
-  gtk_box_pack_end (GTK_BOX (GTK_FILE_SELECTION (cmd_sv.dialog)->main_vbox),
+  gtk_box_pack_end (GTK_BOX (GTK_FILE_SELECTION (dialog)->main_vbox),
                                                         vbox, FALSE, FALSE, 0);
   name = g_path_get_basename (orz_mdi_get_file (ORZ_MDI (mdi), page_num));
   tmp = root ? g_strconcat (root, name, NULL)
-             : g_strconcat (save_path, G_DIR_SEPARATOR_S, name, NULL);
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (cmd_sv.dialog), tmp);
+             : g_build_filename (save_path, name, NULL);
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), tmp);
+  g_free (root);
   g_free (tmp);
   g_free (name);
-  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (cmd_sv.dialog));
-  gtk_widget_show (cmd_sv.dialog);
-  gtk_grab_add (cmd_sv.dialog);
-  gtk_main ();
-#endif /* not GTK_CHECK_VERSION(2,4,0) */
-  if (cmd_sv.file)
+  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (dialog));
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
     {
-      gchar *text, *utf8str;
+      file = g_strdup (gtk_file_selection_get_filename
+                                                (GTK_FILE_SELECTION (dialog)));
+      root = g_strdup (gtk_entry_get_text
+                                    (GTK_ENTRY (GTK_COMBO (combo0)->entry)));
+      charset = orz_charset_get_charset (ORZ_CHARSET (combo1));
+      filekind = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio0))
+                                            ? FILE_KIND_DB : FILE_KIND_LIST;
+      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio3)))
+        playlist = 1;
+      else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio4)))
+        playlist = 2;
+      else
+        playlist = 0;
+    }
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
+  else
+    {
+      file = root = charset = NULL;
+      filekind = FILE_KIND_DB;
+      playlist = 0;
+    }
+  gtk_widget_destroy (dialog);
+  if (file)
+    {
+      gboolean result = TRUE;
 
-      utf8str = g_filename_to_utf8 (cmd_sv.file, -1, NULL, NULL, NULL);
-      text = g_strdup_printf (_("%s\nThis file is existed.\n\nOver write?"),
-                                                                    utf8str);
-      g_free (utf8str);
-      if ((!fileio_isfile (cmd_sv.file)
-                                || misc_message_box ("Melody maid", text, 1,
-                                            _("_Yes"), _("_No"), NULL) == 0)
-            && (cmd_sv.filekind == FILE_KIND_DB
-            ? file_save_db (cmd_sv.file, mmaid, cmd_sv.root, cmd_sv.charset)
-            : file_save_list (cmd_sv.file, mmaid, cmd_sv.root, cmd_sv.charset,
-                                                            cmd_sv.playlist)))
+      if (fileio_isfile (file))
+        {
+          gchar *utf8str;
+
+          utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+          dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                    GTK_MESSAGE_INFO, GTK_BUTTONS_YES_NO,
+                    _("%s\nThis file is existed.\n\nOver write?"), utf8str);
+          g_free (utf8str);
+          result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES;
+          gtk_widget_destroy (dialog);
+        }
+      if (result && (filekind == FILE_KIND_DB
+                    ? file_save_db (file, mmaid, root, charset)
+                    : file_save_list (file, mmaid, root, charset, playlist)))
         {
           gchar *path;
 
           g_free (save_path);
-          path = fileio_get_full_path (cmd_sv.file);
+          path = fileio_get_full_path (file);
           save_path = g_path_get_dirname (path);
           g_free (path);
           /* ja:キャラクターセット */
           g_free (mmaid->charset);
-          mmaid->charset = g_strdup (cmd_sv.charset);
-          if (cmd_sv.filekind == FILE_KIND_DB || cmd_sv.playlist != 2)
+          mmaid->charset = g_strdup (charset);
+          if (filekind == FILE_KIND_DB || playlist != 2)
             {
               /* ja:諸設定 */
               orz_mdi_set_created (ORZ_MDI (mdi), page_num, FALSE);
               orz_mdi_set_edited (ORZ_MDI (mdi), page_num, FALSE);
-              orz_mdi_set_file (ORZ_MDI (mdi), page_num, cmd_sv.file);
+              orz_mdi_set_file (ORZ_MDI (mdi), page_num, file);
             }
-          mmaid->filekind = cmd_sv.filekind;
-          if (cmd_sv.filekind == FILE_KIND_LIST)
-            playlist = mmaid->playlist = cmd_sv.playlist;
-          orz_history_add_file (ORZ_HISTORY (history), cmd_sv.file);
+          mmaid->filekind = filekind;
+          if (filekind == FILE_KIND_LIST)
+            playlist = mmaid->playlist = playlist;
+          orz_history_add_file (ORZ_HISTORY (history), file);
         }
-      g_free (text);
+      g_free (file);
+      g_free (root);
+      g_free (charset);
     }
-  g_free (cmd_sv.file);
-  g_free (cmd_sv.root);
-  g_free (cmd_sv.charset);
-  g_free (root);
 }
 
 
@@ -974,7 +957,7 @@ command_search_recursive (const gchar *path,
           if (name[0] == '.')
             continue;
 #endif /* not G_OS_WIN32 */
-          file = g_strconcat (path, G_DIR_SEPARATOR_S, name, NULL);
+          file = g_build_filename (path, name, NULL);
           if (g_file_test (file, G_FILE_TEST_IS_DIR))
             {
               command_search_recursive (file, dialog);
@@ -1097,17 +1080,8 @@ command_search (GtkWidget *widget,
     }
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
     path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-  gtk_widget_destroy (dialog);
 #else /* not GTK_CHECK_VERSION(2,4,0) */
   dialog = gtk_file_selection_new (_("Browse"));
-  g_signal_connect (G_OBJECT (dialog), "destroy",
-                                            G_CALLBACK (gtk_main_quit), NULL);
-  g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (dialog)->ok_button),
-                            "clicked", G_CALLBACK (command_open_ok), dialog);
-  g_signal_connect_swapped
-                        (G_OBJECT (GTK_FILE_SELECTION (dialog)->cancel_button),
-            "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
-  g_object_set_data (G_OBJECT (dialog), "user_data", &path);
   if (path)
     {
       if (path[0] != '\0')
@@ -1129,10 +1103,11 @@ command_search (GtkWidget *widget,
   gtk_widget_hide
             (gtk_widget_get_parent (GTK_FILE_SELECTION (dialog)->file_list));
   gtk_widget_hide (GTK_FILE_SELECTION (dialog)->selection_entry);
-  gtk_widget_show (dialog);
-  gtk_grab_add (dialog);
-  gtk_main ();
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
+    path = g_strdup (gtk_file_selection_get_filename
+                                                (GTK_FILE_SELECTION (dialog)));
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
+  gtk_widget_destroy (dialog);
 
   if (!path)
     return;
@@ -1343,22 +1318,6 @@ command_all (GtkWidget *widget,
 }
 
 
-static void
-command_conf_ok (GtkWidget *widget,
-                 GtkWidget *dialog)
-{
-  orz_history_set_num (ORZ_HISTORY (history),
-                                    orz_conf_get_history (ORZ_CONF (dialog)));
-  newfile = orz_conf_get_newfile (ORZ_CONF (dialog));
-  second = orz_conf_get_second (ORZ_CONF (dialog));
-  orz_conf_get_size (ORZ_CONF (dialog), &def_width, &def_height);
-  n_pos = orz_conf_get_tab (ORZ_CONF (dialog));
-  gtk_widget_destroy (dialog);
-  /* ja:ノートの位置 */
-  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (mdi), n_pos);
-}
-
-
 void
 command_conf (GtkWidget *widget,
               gpointer   user_data)
@@ -1366,21 +1325,25 @@ command_conf (GtkWidget *widget,
   GtkWidget *dialog;
 
   dialog = orz_conf_new ();
-  g_signal_connect (G_OBJECT (dialog), "destroy",
-                                            G_CALLBACK (gtk_main_quit), NULL);
-  g_signal_connect (G_OBJECT (ORZ_CONF (dialog)->ok_button),
-                            "clicked", G_CALLBACK (command_conf_ok), dialog);
-  g_signal_connect_swapped (G_OBJECT (ORZ_CONF (dialog)->cancel_button),
-            "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
   orz_conf_set_history (ORZ_CONF (dialog),
                                 orz_history_get_num (ORZ_HISTORY (history)));
   orz_conf_set_newfile (ORZ_CONF (dialog), newfile);
   orz_conf_set_second (ORZ_CONF (dialog), second);
   orz_conf_set_size (ORZ_CONF (dialog), def_width, def_height);
   orz_conf_set_tab (ORZ_CONF (dialog), n_pos);
-  gtk_widget_show (dialog);
-  gtk_grab_add (dialog);
-  gtk_main ();
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
+    {
+      orz_history_set_num (ORZ_HISTORY (history),
+                                    orz_conf_get_history (ORZ_CONF (dialog)));
+      newfile = orz_conf_get_newfile (ORZ_CONF (dialog));
+      second = orz_conf_get_second (ORZ_CONF (dialog));
+      orz_conf_get_size (ORZ_CONF (dialog), &def_width, &def_height);
+      n_pos = orz_conf_get_tab (ORZ_CONF (dialog));
+      /* ja:ノートの位置 */
+      gtk_notebook_set_tab_pos (GTK_NOTEBOOK (mdi), n_pos);
+    }
+  gtk_widget_destroy (dialog);
 }
 
 
@@ -1392,17 +1355,6 @@ command_genre (GtkWidget *widget,
 }
 
 
-static void
-command_charset_ok (GtkWidget *widget,
-                    GtkWidget *dialog)
-{
-  g_free (charset_list);
-  charset_list = orz_charlist_get_charlist (ORZ_CHARLIST (dialog));
-  charlist_renewal_all (charset_list);
-  gtk_widget_destroy (dialog);
-}
-
-
 void
 command_charset (GtkWidget *widget,
                  gpointer   user_data)
@@ -1410,16 +1362,15 @@ command_charset (GtkWidget *widget,
   GtkWidget *dialog;
 
   dialog = orz_charlist_new ();
-  g_signal_connect (G_OBJECT (dialog), "destroy",
-                                            G_CALLBACK (gtk_main_quit), NULL);
-  g_signal_connect (G_OBJECT (ORZ_CHARLIST (dialog)->ok_button),
-                        "clicked", G_CALLBACK (command_charset_ok), dialog);
-  g_signal_connect_swapped (G_OBJECT (ORZ_CHARLIST (dialog)->cancel_button),
-            "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
   orz_charlist_set_charlist (ORZ_CHARLIST (dialog), charset_list);
-  gtk_widget_show (dialog);
-  gtk_grab_add (dialog);
-  gtk_main ();
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
+    {
+      g_free (charset_list);
+      charset_list = orz_charlist_get_charlist (ORZ_CHARLIST (dialog));
+      charlist_renewal_all (charset_list);
+    }
+  gtk_widget_destroy (dialog);
 }
 
 
