@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "abort.h"
 #include "file.h"
@@ -23,7 +23,6 @@
 #include "sigfile.h"
 #include "sigmain.h"
 #include "misc/fileio.h"
-#include "misc/misc.h"
 #include "orz/orzhistory.h"
 #include "orz/orzmdi.h"
 
@@ -46,7 +45,7 @@ typedef struct _MetaInfo
 } MetaInfo;
 
 
-static const gchar irivdb_header0[] = {
+const static gchar irivdb_header0[] = {
   0x69, 0x52, 0x69, 0x76, 0x44, 0x42, 0x20, 0x56,
   0x65, 0x72, 0x20, 0x30, 0x2e, 0x31, 0x32, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -56,7 +55,7 @@ static const gchar irivdb_header0[] = {
   0x42, 0x20, 0x46, 0x69, 0x6c, 0x65, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
 };
-static const gchar irivdb_header1[] = {
+const static gchar irivdb_header1[] = {
   0x44, 0x65, 0x73, 0x69, 0x67, 0x6e, 0x65, 0x64,
   0x20, 0x62, 0x79, 0x20, 0x69, 0x52, 0x69, 0x76,
   0x65, 0x72, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -70,26 +69,26 @@ static const gchar irivdb_header1[] = {
 
 /*  ja:パスのキャラクターセットを変換する
        file,ファイル名
-       leng,バイト数
+     length,バイト数
     charset,キャラクターセット
-        err,TRUE:エラーなし,FALSE:強行
+        err,TRUE:エラー中断,FALSE:エラー強行
         RET,変換されたファイル名,NULL:エラー                                */
 static gchar *
-file_path_convert (const gchar  *file,
-                   const gssize  leng,
-                   const gchar  *charset,
-                   gboolean     *err)
+file_path_convert (const gchar    *file,
+                   const gsize     length,
+                   const gchar    *charset,
+                   const gboolean  err)
 {
-  gchar *result = NULL, *tmp;
-  gsize bytes_done = 0, length;
+  gchar *result = NULL;
+  gsize bytes_done = 0, leng;
 
-  if (err)
-    *err = TRUE;
-  if (!file || leng == 0 || !charset)
+  if (!file || length == 0 || !charset)
     return NULL;
-  length = leng < 0 ? g_strlen (file) : leng;
-  while (bytes_done < length)
+  leng = MIN (length == (gsize)-1 ? g_strlen (file) : length, 260);
+  while (bytes_done < leng)
     {
+      gchar *tmp;
+
       if (file[bytes_done] == '\\')
         {
           tmp = result ? g_strconcat (result, G_DIR_SEPARATOR_S, NULL)
@@ -101,7 +100,7 @@ file_path_convert (const gchar  *file,
           gchar *str = NULL;
           gsize len = 1;
 
-          for (len = 1; bytes_done + len <= length; len++)
+          for (len = 1; bytes_done + len <= leng; len++)
             {
               str = g_convert (file + bytes_done, len, "UTF-8", charset,
                                                             NULL, NULL, NULL);
@@ -114,12 +113,15 @@ file_path_convert (const gchar  *file,
               bytes_done += len;
               g_free (str);
             }
-          else
+          else if (!err)
             {
               tmp = result ? g_strconcat (result, "?", NULL) : g_strdup ("?");
               bytes_done++;
-              if (err)
-                *err = FALSE;
+            }
+          else
+            {
+              g_free (result);
+              return NULL;
             }
         }
       g_free (result);
@@ -146,8 +148,8 @@ file_is_type (const gchar *file)
   if (!fio)
     return FILE_KIND_ERROR;
   if (fileio_read (fio, header, sizeof (header)) == sizeof (header)
-    && !g_memcmp (irivdb_header0, header, sizeof (irivdb_header0))
-    && !g_memcmp (irivdb_header1, header + 0xa080, sizeof (irivdb_header1)))
+    && g_memcmp (irivdb_header0, header, sizeof (irivdb_header0)) == 0
+    && g_memcmp (irivdb_header1, header + 0xa080, sizeof (irivdb_header1))== 0)
     {
       fileio_close (fio);
       return FILE_KIND_DB;
@@ -173,33 +175,35 @@ file_is_type (const gchar *file)
   fileio_close (fio);
   /* ja:キャラクターセット変換 */
   dir = g_path_get_dirname (file);
-  ary = g_strsplit (buf, "\r\n", G_MAXINT);
-  charset = g_strsplit (charset_list, ",", G_MAXINT);
+  ary = g_strsplit (buf, "\r\n", 0);
+  charset = g_strsplit (charset_list, ",", 0);
   g_free (buf);
   for (i = 0; charset[i]; i++)
     {
       gint j;
 
       for (j = 0; ary[j]; j++)
-        if (*(ary[j]) != '\0' && *(ary[j]) != '#'
-                                        && *(ary[j]) != G_SEARCHPATH_SEPARATOR)
+        if (*(ary[j]) != '\0' && *(ary[j]) != '#' && *(ary[j]) != ';')
           {
-            gboolean err;
+            gboolean result;
             gchar *str;
 
-            str = file_path_convert (ary[j], -1, charset[i], &err);
-            if (str && err)
+            str = file_path_convert (ary[j], -1, charset[i], TRUE);
+            if (str)
               {
                 gchar *path;
 
                 path = g_path_is_absolute (str) ? g_strdup (str)
-                            : g_strconcat (dir, G_DIR_SEPARATOR_S, str, NULL);
-                if (!fileio_isfile (path))
-                  err = FALSE;
+                                        : g_build_filename (dir, str, NULL);
+                result = fileio_isfile (path);
                 g_free (path);
+                g_free (str);
               }
-            g_free (str);
-            if (!err)
+            else
+              {
+                result = FALSE;
+              }
+            if (!result)
               break;
           }
       if (!ary[j])
@@ -272,7 +276,7 @@ file_open_id3tag (const gchar *file,
       comment[28] = comment[29] = ' ';
     }
   /* ja:キャラクターセット変換 */
-  charset = g_strsplit (charlist, ",", G_MAXINT);
+  charset = g_strsplit (charlist, ",", 0);
   for (i = 0; charset[i]; i++)
     {
       id3tag->title   = g_convert (title,   sizeof (title),
@@ -384,9 +388,9 @@ file_open_db (const gchar *file,
   fio = fileio_open (file, FILEIO_ACCESS_READ, FILEIO_SHARE_READ,
                                                     FILEIO_MODE_OPEN_EXISTING);
   if (!fio || fileio_read (fio, header, sizeof (header)) != sizeof (header)
-        || g_memcmp (irivdb_header0, header, sizeof (irivdb_header0))
-        || g_memcmp (irivdb_header1, header + 0xa080, sizeof (irivdb_header1))
-        || (entries = GINT32_FROM_LE (*(gint32 *)(header + 0x0040))) > 10240)
+    || g_memcmp (irivdb_header0, header, sizeof (irivdb_header0)) != 0
+    || g_memcmp (irivdb_header1, header + 0xa080, sizeof (irivdb_header1)) != 0
+    || (entries = GINT32_FROM_LE (*(gint32 *)(header + 0x0040))) > 10240)
     entries = 0;/* ja:ヘッタにエラーがある */
   /* ja:ヘッタのPaddingをチェック */
   if (entries)
@@ -437,7 +441,7 @@ file_open_db (const gchar *file,
   if (fio)
     fileio_close (fio);
   /* ja:キャラクターセット変換 */
-  charset = g_strsplit (mmaid->charset, ",", G_MAXINT);
+  charset = g_strsplit (mmaid->charset, ",", 0);
   g_free (mmaid->charset);
   mmaid->charset = NULL;
   for (i = 0; charset[i]; i++)
@@ -447,16 +451,15 @@ file_open_db (const gchar *file,
       g_memset (mi_c, 0, sizeof (mi_c));
       for (j = 0; j < entries; j++)
         {
-          gboolean err;
           gchar *str;
 
           while (gtk_events_pending ())
             gtk_main_iteration ();
           str = file_path_convert (mi_r[j].str[0], mi_r[j].len[0] - 1,
-                                                            charset[i], &err);
-          mi_c[j].str[0] = str;
-          if (!str || !err)
+                                                            charset[i], TRUE);
+          if (!str)
             break;
+          mi_c[j].str[0] = str;
           for (k = 1; k < 5; k++)
             {
               str = g_convert (mi_r[j].str[k], mi_r[j].len[k] - 1,
@@ -487,7 +490,7 @@ file_open_db (const gchar *file,
             gtk_main_iteration ();
           mi_c[i].str[0]
                     = file_path_convert (mi_r[i].str[0], mi_r[i].len[0] - 1,
-                                                            charset[0], NULL);
+                                                            charset[0], FALSE);
           if (!mi_c[i].str[0])
             break;
           for (j = 1; j < 5; j++)
@@ -540,6 +543,26 @@ file_open_db (const gchar *file,
       id3tag->album   = mi_c[i].str[METAINFO_ALBUM];
       id3tag->track   = -1;
       id3tag->genre   = mi_c[i].str[METAINFO_GENRE];
+      if (id3tag->title[0] == '\0')
+        {
+          g_free (id3tag->title);
+          id3tag->title = NULL;
+        }
+      if (id3tag->artist[0] == '\0')
+        {
+          g_free (id3tag->artist);
+          id3tag->artist = NULL;
+        }
+      if (id3tag->album[0] == '\0')
+        {
+          g_free (id3tag->album);
+          id3tag->album = NULL;
+        }
+      if (id3tag->genre[0] == '\0')
+        {
+          g_free (id3tag->genre);
+          id3tag->genre = NULL;
+        }
       mm_list_append (MM_LIST (mmaid->list), id3tag);
     }
   g_free (dir);
@@ -585,8 +608,8 @@ file_open_list (const gchar *file,
   fileio_close (fio);
   /* ja:キャラクターセット変換 */
   dir = g_path_get_dirname (file);
-  ary = g_strsplit (buf, "\r\n", G_MAXINT);
-  charset = g_strsplit (mmaid->charset, ",", G_MAXINT);
+  ary = g_strsplit (buf, "\r\n", 0);
+  charset = g_strsplit (mmaid->charset, ",", 0);
   g_free (mmaid->charset);
   g_free (buf);
   for (i = 0; charset[i]; i++)
@@ -595,39 +618,32 @@ file_open_list (const gchar *file,
 
       plist = 0;
       for (j = 0; ary[j]; j++)
-        if (*(ary[j]) != '\0' && *(ary[j]) != '#'
-                                        && *(ary[j]) != G_SEARCHPATH_SEPARATOR)
+        if (*(ary[j]) != '\0' && *(ary[j]) != '#' && *(ary[j]) != ';')
           {
-            gboolean err;
-            gchar *str;
+            gchar *str, *path;
 
-            str = file_path_convert (ary[j], -1, charset[i], &err);
-            if (str && err)
+            str = file_path_convert (ary[j], -1, charset[i], TRUE);
+            if (!str)
+              break;
+            if (g_path_is_absolute (str))
               {
-                gchar *path;
-
-                if (g_path_is_absolute (str))
-                  {
-                    path = g_strdup (str);
-                    plist = 1;
-                  }
-                else
-                  {
-                    path = g_strconcat (dir, G_DIR_SEPARATOR_S, str, NULL);
-                  }
-                if (fileio_isfile (path))
-                  {
-                    glist = g_list_append (glist, path);
-                  }
-                else
-                  {
-                    g_free (path);
-                    err = FALSE;
-                  }
+                path = g_strdup (str);
+                plist = 1;
+              }
+            else
+              {
+                path = g_build_filename (dir, str, NULL);
               }
             g_free (str);
-            if (!err)
-              break;
+            if (fileio_isfile (path))
+              {
+                glist = g_list_append (glist, path);
+              }
+            else
+              {
+                g_free (path);
+                break;
+              }
           }
       if (!ary[j])
         break;
@@ -686,7 +702,7 @@ file_open_edit (const gchar *file)
 
       /* ja:新規 */
       filename = g_strdup_printf ("new%04d.idb", fcount++ % 1000);
-      charset = g_strsplit (charset_list, ",", G_MAXINT);
+      charset = g_strsplit (charset_list, ",", 0);
       mmaid->charset = g_strdup (charset[0]);
       g_strfreev (charset);
       mmaid->filekind = FILE_KIND_DB;
@@ -912,7 +928,7 @@ file_save_db (const gchar *file,
       while (gtk_events_pending ())
         gtk_main_iteration ();
       id3tag = mm_list_get_nth (MM_LIST (mmaid->list), i);
-      ary = g_strsplit (id3tag->file + length, G_DIR_SEPARATOR_S, G_MAXINT);
+      ary = g_strsplit (id3tag->file + length, G_DIR_SEPARATOR_S, 0);
       for (j = 0; ary[j]; j++)
         {
           tmp = g_convert (ary[j], -1, charset, "UTF-8", NULL, NULL, NULL);
@@ -1003,8 +1019,16 @@ file_save_db (const gchar *file,
               g_free (text);
               text = tmp;
             }
-          misc_message_box ("Melody maid", text, 0, _("OK"), NULL);
+          gtk_grab_remove (dialog);
+          gtk_widget_destroy (dialog);
+          dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                            text);
           g_free (text);
+          gtk_dialog_run (GTK_DIALOG (dialog));
+          gtk_widget_destroy (dialog);
+          dialog = NULL;
           userbreak = FALSE;
         }
       g_free (file);
@@ -1016,23 +1040,35 @@ file_save_db (const gchar *file,
   fio = userbreak ? fileio_open (file,
                         FILEIO_ACCESS_READ | FILEIO_ACCESS_WRITE,
                         FILEIO_SHARE_READ, FILEIO_MODE_CREATE_ALWAYS) : NULL;
+  /* ja:終了処理 */
+  if (dialog)
+    {
+      gtk_grab_remove (dialog);
+      gtk_widget_destroy (dialog);
+    }
   if (!fio)
     {
-      misc_message_box ("Melody maid", _("Can not open file"),
-                                                            0, _("OK"), NULL);
+      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                            _("Can not open file"));
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
       userbreak = FALSE;
     }
   if (fio && fileio_write (fio, buf, (gsize)offset) != offset)
     {
-      misc_message_box ("Melody maid", _("Can not write"), 0, _("OK"), NULL);
+      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                            _("Can not write"));
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
       userbreak = FALSE;
     }
   if (fio)
     fileio_close (fio);
   g_free (buf);
-  /* ja:終了処理 */
-  gtk_grab_remove (dialog);
-  gtk_widget_destroy (dialog);
   return userbreak;
 }
 
@@ -1052,13 +1088,13 @@ file_save_list (const gchar *file,
                 const gint   playlist)
 {
   FileIO *fio;
+  GtkWidget *dialog;
 
   fio = fileio_open (file, FILEIO_ACCESS_READ | FILEIO_ACCESS_WRITE,
                                 FILEIO_SHARE_READ, FILEIO_MODE_CREATE_ALWAYS);
   if (fio)
     {
       gint i, count;
-      GtkWidget *dialog;
 
       /* ja:ダイアログボックス */
       userbreak = TRUE;
@@ -1107,7 +1143,7 @@ file_save_list (const gchar *file,
           tmp = g_strconcat (element, "\r\n", NULL);
           g_free (element);
           element = NULL;
-          ary = g_strsplit (tmp, G_DIR_SEPARATOR_S, G_MAXINT);
+          ary = g_strsplit (tmp, G_DIR_SEPARATOR_S, 0);
           g_free (tmp);
           for (j = 0; ary[j]; j++)
             {
@@ -1117,15 +1153,17 @@ file_save_list (const gchar *file,
                                         charset, "UTF-8", NULL, NULL, NULL);
               if (!name)
                 {
-                  gchar *text;
-
                   g_free (element);
-                  text = g_strdup_printf
-                                        (_("Conversion from character encoding"
-                                        " 'UTF-8' to '%s' is not supported"),
-                                                            charset);
-                  misc_message_box ("Melody maid", text, 0, _("OK"), NULL);
-                  g_free (text);
+                  gtk_grab_remove (dialog);
+                  gtk_widget_destroy (dialog);
+                  dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                            _("Conversion from character encoding"
+                              " 'UTF-8' to '%s' is not supported"), charset);
+                  gtk_dialog_run (GTK_DIALOG (dialog));
+                  gtk_widget_destroy (dialog);
+                  dialog = NULL;
                   userbreak = FALSE;
                   break;
                 }
@@ -1149,22 +1187,36 @@ file_save_list (const gchar *file,
               leng = g_strlen (element) * sizeof (gchar);
               if (fileio_write (fio, element, leng) != leng)
                 {
-                  misc_message_box ("Melody maid",
-                                        _("Can not write"), 0, _("OK"), NULL);
+                  gtk_grab_remove (dialog);
+                  gtk_widget_destroy (dialog);
+                  dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                            _("Can not write"));
+                  gtk_dialog_run (GTK_DIALOG (dialog));
+                  gtk_widget_destroy (dialog);
+                  dialog = NULL;
                   userbreak = FALSE;
                 }
               g_free (element);
             }
         }
       /* ja:終了処理 */
-      gtk_grab_remove (dialog);
-      gtk_widget_destroy (dialog);
+      if (dialog)
+        {
+          gtk_grab_remove (dialog);
+          gtk_widget_destroy (dialog);
+        }
       fileio_close (fio);
     }
   else
     {
-      misc_message_box ("Melody maid", _("Can not open file"),
-                                                            0, _("OK"), NULL);
+      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                            _("Can not open file"));
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
       userbreak = FALSE;
     }
   return userbreak;
