@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "command.h"
 #include "general.h"
@@ -193,16 +193,27 @@ prompt_close (MmaidWindow *mmaid)
   page = orz_mdi_get_page_from_data (ORZ_MDI (mdi), mmaid);
   if (orz_mdi_get_edited (ORZ_MDI (mdi), page))
     {
-      gchar *text, *utf8str;
+      gchar *utf8str;
+      gint result;
+      GtkWidget *dialog;
 
       utf8str = g_filename_to_utf8
                 (orz_mdi_get_file (ORZ_MDI (mdi), page), -1, NULL, NULL, NULL);
-      text = g_strdup_printf (_("File %s was edited.\nSave?"), utf8str);
+      dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_INFO, GTK_BUTTONS_NONE,
+                            _("File %s was edited.\nSave?"), utf8str);
       g_free (utf8str);
-      switch (misc_message_box ("Melody maid", text, 0,
-                                    _("_Yes"), _("_No"), _("Cancel"), NULL))
+      gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                                        GTK_STOCK_YES, GTK_RESPONSE_YES,
+                                        GTK_STOCK_NO, GTK_RESPONSE_NO,
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                        NULL);
+      result = gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      switch (result)
         {
-          case 0:
+          case GTK_RESPONSE_YES:
             {
               gint page_num;
 
@@ -217,12 +228,12 @@ prompt_close (MmaidWindow *mmaid)
               if (!orz_mdi_get_edited (ORZ_MDI (mdi), page))
                 break;
             }
-          case 2:
-          case -1:
-            g_free (text);
+            return FALSE;
+          case GTK_RESPONSE_NO:
+            break;
+          default:
             return FALSE;
         }
-      g_free (text);
     }
   return TRUE;
 }
