@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "mm_list.h"
 #include "mm_dialog.h"
@@ -23,15 +23,15 @@
 
 static void mm_dialog_class_init (MmDialogClass *klass);
 static void mm_dialog_init       (MmDialog      *dialog);
-static void mm_dialog_close      (MmDialog      *dialog);
+static void mm_dialog_escape     (MmDialog      *dialog);
 
 
-static GtkWindowClass *parent_class = NULL;
+static GtkDialogClass *parent_class = NULL;
 
 
 enum
 {
-  CLOSE_SIGNAL,
+  ESCAPE_SIGNAL,
   LAST_SIGNAL
 };
 
@@ -49,7 +49,7 @@ mm_dialog_get_type (void)
 
   if (!type)
     {
-      static const GTypeInfo info =
+      const static GTypeInfo info =
       {
         sizeof (MmDialogClass),
         NULL,               /* base_init */
@@ -62,7 +62,7 @@ mm_dialog_get_type (void)
         (GInstanceInitFunc)mm_dialog_init,
       };
 
-      type = g_type_register_static (GTK_TYPE_WINDOW, "MmDialog", &info, 0);
+      type = g_type_register_static (GTK_TYPE_DIALOG, "MmDialog", &info, 0);
     }
 
   return type;
@@ -83,24 +83,24 @@ mm_dialog_class_init (MmDialogClass *klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  klass->close = mm_dialog_close;
+  klass->escape = mm_dialog_escape;
 
-  mm_dialog_signals[CLOSE_SIGNAL]
-                = g_signal_new ("close",
+  mm_dialog_signals[ESCAPE_SIGNAL]
+                = g_signal_new ("escape",
                                 G_TYPE_FROM_CLASS (klass),
                                 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-                                G_STRUCT_OFFSET (MmDialogClass, close),
+                                G_STRUCT_OFFSET (MmDialogClass, escape),
                                 NULL, NULL,
                                 g_cclosure_marshal_VOID__VOID,
                                 G_TYPE_NONE, 0);
 
   binding_set = gtk_binding_set_by_class (klass);
-  gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0, "close", 0);
+  gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0, "escape", 0);
 }
 
 
 static void
-mm_dialog_close (MmDialog *dialog)
+mm_dialog_escape (MmDialog *dialog)
 {
   if (GTK_WIDGET_VISIBLE (dialog->abort_button)
                             && GTK_WIDGET_SENSITIVE (dialog->abort_button))
@@ -141,7 +141,7 @@ mm_dialog_delete (GtkWidget *widget,
 static void
 mm_dialog_init (MmDialog *dialog)
 {
-  GtkWidget *scroll, *hbox, *vbox;
+  GtkWidget *scroll;
 
   /* ja:メインウインドウ */
   gtk_window_set_default_size (GTK_WINDOW (dialog),
@@ -149,12 +149,12 @@ mm_dialog_init (MmDialog *dialog)
   g_signal_connect (G_OBJECT (dialog), "delete-event",
                                         G_CALLBACK (mm_dialog_delete), NULL);
   /* ja:ボタン */
-  dialog->abort_button  = gtk_button_new_with_mnemonic (_("_Abort"));
-  dialog->ok_button     = gtk_button_new_with_label    (_("OK"));
-  dialog->cancel_button = gtk_button_new_with_label    (_("Cancel"));
-  GTK_WIDGET_SET_FLAGS (dialog->abort_button,  GTK_CAN_DEFAULT);
-  GTK_WIDGET_SET_FLAGS (dialog->ok_button,     GTK_CAN_DEFAULT);
-  GTK_WIDGET_SET_FLAGS (dialog->cancel_button, GTK_CAN_DEFAULT);
+  dialog->abort_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+                                        GTK_STOCK_STOP, GTK_RESPONSE_REJECT);
+  dialog->ok_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+                                        GTK_STOCK_OK, GTK_RESPONSE_OK);
+  dialog->cancel_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
   /* ja:リスト */
   dialog->list = mm_list_new ();
   /* ja:スクロールウインドウ */
@@ -162,20 +162,12 @@ mm_dialog_init (MmDialog *dialog)
   gtk_container_add (GTK_CONTAINER (scroll), dialog->list);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_container_set_border_width (GTK_CONTAINER (scroll), SPACING);
   /* ja:フレームとボックス */
-  vbox = gtk_vbox_new (FALSE, SPACING);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
-  gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
-  hbox = gtk_hbox_new (FALSE, SPACING);
-  gtk_box_pack_end (GTK_BOX (hbox), dialog->cancel_button, FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (hbox), dialog->ok_button,     FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (hbox), dialog->abort_button,  FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (dialog), vbox);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), scroll);
 
   /* ja:表示 */
-  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
-  gtk_widget_show_all (vbox);
+  gtk_widget_show_all (scroll);
   gtk_widget_hide (dialog->abort_button);
   gtk_widget_hide (dialog->ok_button);
   gtk_widget_hide (dialog->cancel_button);
