@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "general.h"
 #include "mm_list.h"
@@ -59,7 +59,7 @@ mm_list_get_type (void)
 
   if (!type)
     {
-      static const GTypeInfo info =
+      const static GTypeInfo info =
       {
         sizeof (MmListClass),
         NULL,               /* base_init */
@@ -190,7 +190,8 @@ mm_list_clicked (GtkTreeViewColumn *column,
   gint i, count, c_visible, c_sensitive, c_toggle;
   GtkTreeIter iter;
 
-  c_toggle = (gint)g_object_get_data (G_OBJECT (column), "user_data");
+  c_toggle = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column),
+                                                                "user_data"));
   switch (c_toggle)
     {
       case MM_LIST_ADD_TOGGLE:
@@ -252,7 +253,7 @@ mm_list_toggled (GtkCellRendererToggle *cell,
   GtkTreeIter iter;
   GtkTreePath *path;
 
-  c = (gint)g_object_get_data (G_OBJECT (cell), "user_data");
+  c = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "user_data"));
   path = gtk_tree_path_new_from_string (path_string);
   gtk_tree_model_get_iter (GTK_TREE_MODEL (list->store), &iter, path);
   gtk_tree_path_free (path);
@@ -279,7 +280,7 @@ mm_list_edited (GtkCellRendererText *cell,
   GtkTreePath *path;
   ID3Tag *id3tag;
 
-  c = (gint)g_object_get_data (G_OBJECT (cell), "user_data");
+  c = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "user_data"));
   switch (c)
     {
       case MM_LIST_FILE:
@@ -392,7 +393,8 @@ mm_list_init (MmList *list)
 
       renderer = gtk_cell_renderer_text_new ();
       g_signal_connect (renderer, "edited", G_CALLBACK (mm_list_edited), list);
-      g_object_set_data (G_OBJECT (renderer), "user_data", (gpointer)(i + 1));
+      g_object_set_data (G_OBJECT (renderer), "user_data",
+                                                    GINT_TO_POINTER (i + 1));
       column = gtk_tree_view_column_new_with_attributes (_(title[i]), renderer,
                 "text", i + 1,
                 "editable", i == 0 ? MM_LIST_EDITABLE_FILE : MM_LIST_EDITABLE,
@@ -402,7 +404,8 @@ mm_list_init (MmList *list)
       gtk_tree_view_append_column (GTK_TREE_VIEW (list), column);
       g_signal_connect (G_OBJECT (column), "clicked",
                                         G_CALLBACK (mm_list_clicked), list);
-      g_object_set_data (G_OBJECT (column), "user_data", (gpointer)(i + 1));
+      g_object_set_data (G_OBJECT (column), "user_data",
+                                                    GINT_TO_POINTER (i + 1));
     }
   select = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
   gtk_tree_selection_set_mode (select, GTK_SELECTION_MULTIPLE);
@@ -668,9 +671,13 @@ mm_list_change (MmList     *list,
         {
           case MM_LIST_ID3TAG:
             {
+              gchar *track;
               ID3Tag *id3tag0;
 
               id3tag0 = va_arg (ap, ID3Tag *);
+              track = 0 <= id3tag0->track && id3tag0->track <= 255
+                                   ? g_strdup_printf ("%03d", id3tag0->track)
+                                   : NULL;
               gtk_list_store_set (list->store, &iter,
                   MM_LIST_ID3TAG,  id3tag0,
                   MM_LIST_FILE,    id3tag0->file,
@@ -679,12 +686,11 @@ mm_list_change (MmList     *list,
                   MM_LIST_ALBUM,   id3tag0->album,
                   MM_LIST_YEAR,    id3tag0->year,
                   MM_LIST_COMMENT, id3tag0->comment,
-                  MM_LIST_TRACK,   0 <= id3tag0->track && id3tag0->track <= 255
-                                   ? g_strdup_printf ("%03d", id3tag0->track)
-                                   : NULL,
+                  MM_LIST_TRACK,   track,
                   MM_LIST_GENRE,   id3tag0->genre,
                   MM_LIST_CHARSET, id3tag0->charset,
                   -1);
+              g_free (track);
               g_free (id3tag->file);
               g_free (id3tag->title);
               g_free (id3tag->artist);
@@ -698,48 +704,61 @@ mm_list_change (MmList     *list,
             }
             break;
           case MM_LIST_FILE:
+            g_free (id3tag->file);
             id3tag->file = va_arg (ap, gchar *);
             gtk_list_store_set (list->store, &iter,
                                         MM_LIST_FILE, id3tag->file, -1);
             break;
           case MM_LIST_TITLE:
+            g_free (id3tag->title);
             id3tag->title = va_arg (ap, gchar *);
             gtk_list_store_set (list->store, &iter,
                                         MM_LIST_TITLE, id3tag->title, -1);
             break;
           case MM_LIST_ARTIST:
+            g_free (id3tag->artist);
             id3tag->artist = va_arg (ap, gchar *);
             gtk_list_store_set (list->store, &iter,
                                         MM_LIST_ARTIST, id3tag->artist, -1);
             break;
           case MM_LIST_ALBUM:
+            g_free (id3tag->album);
             id3tag->album = va_arg (ap, gchar *);
             gtk_list_store_set (list->store, &iter,
                                         MM_LIST_ALBUM, id3tag->album, -1);
             break;
           case MM_LIST_YEAR:
+            g_free (id3tag->year);
             id3tag->year = va_arg (ap, gchar *);
             gtk_list_store_set (list->store, &iter,
                                         MM_LIST_YEAR, id3tag->year, -1);
             break;
           case MM_LIST_COMMENT:
+            g_free (id3tag->comment);
             id3tag->comment = va_arg (ap, gchar *);
             gtk_list_store_set (list->store, &iter,
                                         MM_LIST_COMMENT, id3tag->comment, -1);
             break;
           case MM_LIST_TRACK:
-            id3tag->track = va_arg (ap, gint);
-            gtk_list_store_set (list->store, &iter, MM_LIST_TRACK,
-                        0 <= id3tag->track && id3tag->track <= 255
-                            ? g_strdup_printf ("%03d", id3tag->track) : NULL,
-                        -1);
+            {
+              gchar *track;
+
+              id3tag->track = va_arg (ap, gint);
+              track = 0 <= id3tag->track && id3tag->track <= 255
+                            ? g_strdup_printf ("%03d", id3tag->track) : NULL;
+              gtk_list_store_set (list->store, &iter,
+                                        MM_LIST_TRACK, track, -1);
+              g_free (track);
+            }
             break;
           case MM_LIST_GENRE:
+            g_free (id3tag->genre);
             id3tag->genre = va_arg (ap, gchar *);
             gtk_list_store_set (list->store, &iter,
                                         MM_LIST_GENRE, id3tag->genre, -1);
             break;
           case MM_LIST_CHARSET:
+            g_free (id3tag->charset);
             id3tag->charset = va_arg (ap, gchar *);
             gtk_list_store_set (list->store, &iter,
                                         MM_LIST_CHARSET, id3tag->charset, -1);
@@ -1012,8 +1031,9 @@ mm_list_get_cursor (MmList *list,
   if (path)
     gtk_tree_path_free (path);
   if (c)
-    *c = column ? (gint)g_object_get_data (G_OBJECT (column), "user_data")
-                : MM_LIST_INVALID;
+    *c = column
+        ? GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "user_data"))
+        : MM_LIST_INVALID;
 }
 
 
@@ -1039,7 +1059,8 @@ mm_list_set_cursor (MmList     *list,
         {
 
           column = g_list_nth_data (glist, i);
-          if ((gint)g_object_get_data (G_OBJECT (column), "user_data") == c)
+          if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column),
+                                                            "user_data")) == c)
             break;
         }
     }
@@ -1111,8 +1132,9 @@ mm_list_get_cursor_from_pos (MmList     *list,
       if (path)
         gtk_tree_path_free (path);
       if (c)
-        *c = column ? (gint)g_object_get_data (G_OBJECT (column), "user_data")
-                    : MM_LIST_INVALID;
+        *c = column
+        ? GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "user_data"))
+        : MM_LIST_INVALID;
     }
   else
     {
@@ -1218,7 +1240,7 @@ mm_list_show_add (MmList     *list,
       g_signal_connect (G_OBJECT (renderer), "toggled",
                                         G_CALLBACK (mm_list_toggled), list);
       g_object_set_data (G_OBJECT (renderer), "user_data",
-                                            (gpointer)MM_LIST_ADD_TOGGLE);
+                                        GINT_TO_POINTER (MM_LIST_ADD_TOGGLE));
       list->column_add = gtk_tree_view_column_new_with_attributes
                              (_("Add"), renderer,
                             "active",      MM_LIST_ADD_TOGGLE,
@@ -1228,7 +1250,7 @@ mm_list_show_add (MmList     *list,
       g_signal_connect (G_OBJECT (list->column_add), "clicked",
                                         G_CALLBACK (mm_list_clicked), list);
       g_object_set_data (G_OBJECT (list->column_add), "user_data",
-                                            (gpointer)MM_LIST_ADD_TOGGLE);
+                                        GINT_TO_POINTER (MM_LIST_ADD_TOGGLE));
       gtk_tree_view_insert_column (GTK_TREE_VIEW (list), list->column_add, 0);
     }
 }
@@ -1253,7 +1275,7 @@ mm_list_show_refresh (MmList     *list,
       g_signal_connect (G_OBJECT (renderer), "toggled",
                                         G_CALLBACK (mm_list_toggled), list);
       g_object_set_data (G_OBJECT (renderer), "user_data",
-                                            (gpointer)MM_LIST_REFRESH_TOGGLE);
+                                    GINT_TO_POINTER (MM_LIST_REFRESH_TOGGLE));
       list->column_refresh = gtk_tree_view_column_new_with_attributes
                             (_("Refresh"), renderer,
                             "active",      MM_LIST_REFRESH_TOGGLE,
@@ -1263,7 +1285,7 @@ mm_list_show_refresh (MmList     *list,
       g_signal_connect (G_OBJECT (list->column_refresh), "clicked",
                                         G_CALLBACK (mm_list_clicked), list);
       g_object_set_data (G_OBJECT (list->column_refresh), "user_data",
-                                            (gpointer)MM_LIST_REFRESH_TOGGLE);
+                                    GINT_TO_POINTER (MM_LIST_REFRESH_TOGGLE));
       gtk_tree_view_insert_column (GTK_TREE_VIEW (list), list->column_refresh,
                                                     list->column_add ? 1 : 0);
     }
@@ -1289,7 +1311,7 @@ mm_list_show_delete (MmList     *list,
       g_signal_connect (G_OBJECT (renderer), "toggled",
                                         G_CALLBACK (mm_list_toggled), list);
       g_object_set_data (G_OBJECT (renderer), "user_data",
-                                            (gpointer)MM_LIST_DELETE_TOGGLE);
+                                    GINT_TO_POINTER (MM_LIST_DELETE_TOGGLE));
       list->column_delete = gtk_tree_view_column_new_with_attributes
                             (_("Delete"), renderer,
                             "active",      MM_LIST_DELETE_TOGGLE,
@@ -1299,7 +1321,7 @@ mm_list_show_delete (MmList     *list,
       g_signal_connect (G_OBJECT (list->column_delete), "clicked",
                                         G_CALLBACK (mm_list_clicked), list);
       g_object_set_data (G_OBJECT (list->column_delete), "user_data",
-                                            (gpointer)MM_LIST_DELETE_TOGGLE);
+                                    GINT_TO_POINTER (MM_LIST_DELETE_TOGGLE));
       gtk_tree_view_insert_column (GTK_TREE_VIEW (list), list->column_delete,
                 (list->column_add ? 1 : 0) + (list->column_refresh ? 1 : 0));
     }
@@ -1705,7 +1727,7 @@ mm_list_set_status (MmList      *list,
 
       renderer = gtk_cell_renderer_text_new ();
       g_object_set_data (G_OBJECT (renderer), "user_data",
-                                                    (gpointer)MM_LIST_STATUS);
+                                            GINT_TO_POINTER (MM_LIST_STATUS));
       list->column_status = gtk_tree_view_column_new_with_attributes
                                                 (_("Status"), renderer,
                                                 "text", MM_LIST_STATUS, NULL);
@@ -1716,7 +1738,7 @@ mm_list_set_status (MmList      *list,
       g_signal_connect (G_OBJECT (list->column_status), "clicked",
                                         G_CALLBACK (mm_list_clicked), list);
       g_object_set_data (G_OBJECT (list->column_status), "user_data",
-                                                (gpointer)(MM_LIST_STATUS));
+                                            GINT_TO_POINTER (MM_LIST_STATUS));
     }
   else if (list->column_status && !text)
     {
