@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include <locale.h>
 #include "mmaid.h"
@@ -28,8 +28,11 @@
 #include "misc/argument.h"
 #include "misc/charuty.h"
 #include "misc/fileio.h"
+#include "misc/gnugpl.h"
 #include "misc/license.h"
-#include "misc/profile.h"
+#if ! GLIB_CHECK_VERSION(2,6,0)
+# include "misc/profile.h"
+#endif /* not GLIB_CHECK_VERSION(2,6,0) */
 #include "orz/orzaccel.h"
 #include "orz/orzcomm.h"
 #include "orz/orzhistory.h"
@@ -70,7 +73,7 @@ MiscCreateMenuEntry menu_entries[] = {
 {"/file/open",            N_("_Open..."),               "<control>O",      MISC_CREATE_MENU_ITEM,      command_open,    NULL, NULL},
 {"/file/close",           N_("_Close"),                 "<control>W",      MISC_CREATE_MENU_ITEM,      command_close,   NULL, NULL},
 {"/file/save",            N_("_Save"),                  "<control>S",      MISC_CREATE_MENU_ITEM,      command_save,    NULL, NULL},
-{"/file/saveas",          N_("Save _As..."),            "<control>S",      MISC_CREATE_MENU_ITEM,      command_save,    NULL, NULL},
+{"/file/saveas",          N_("Save _As..."),            NULL,              MISC_CREATE_MENU_ITEM,      command_saveas,  NULL, NULL},
 {"/file/reload",          N_("_Reload..."),             "<control>R",      MISC_CREATE_MENU_ITEM,      command_reload,  NULL, NULL},
 {"/file/sep1",            NULL,                         NULL,              MISC_CREATE_MENU_SEPARATOR, NULL,            NULL, NULL},
 {"/file/export",          N_("_Export..."),             "<control>E",      MISC_CREATE_MENU_ITEM,      command_export,  NULL, NULL},
@@ -78,7 +81,7 @@ MiscCreateMenuEntry menu_entries[] = {
 {"/file/exit",            N_("E_xit"),                  "<control>Q",      MISC_CREATE_MENU_ITEM,      command_exit,    NULL, NULL},
 {"/tag",                  N_("_Tag"),                   NULL,              MISC_CREATE_MENU_ITEM,      NULL,            NULL, NULL},
 {"/tag/",                 NULL,                         NULL,              MISC_CREATE_MENU_SHELL,     NULL,            NULL, NULL},
-{"/tag/up",               N_("Up"),                     NULL,              MISC_CREATE_MENU_ITEM,      command_up,      NULL, NULL},
+{"/tag/up",               N_("_Up"),                    NULL,              MISC_CREATE_MENU_ITEM,      command_up,      NULL, NULL},
 {"/tag/down",             N_("_Down"),                  NULL,              MISC_CREATE_MENU_ITEM,      command_down,    NULL, NULL},
 {"/tag/sep1",             NULL,                         NULL,              MISC_CREATE_MENU_SEPARATOR, NULL,            NULL, NULL},
 {"/tag/root",             N_("R_oot..."),               "<control>H",      MISC_CREATE_MENU_ITEM,      command_root,    NULL, NULL},
@@ -128,11 +131,22 @@ int
 main (int   argc,
       char *argv[])
 {
-  gboolean *arg_files, license = FALSE;
-  gchar *section, *key, *utf8str;
-  gint i, j, count;
+  gboolean *arg_files;
+  gchar *group_name, *key, *utf8str, *data, *user_config = NULL;
+#if ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
+  LPITEMIDLIST pidl = NULL;
+#endif /* ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
+#ifndef LOCALEDIR
+  gchar *dirname, *path, *locale;
+# if GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
+  gchar *temp;
+# endif /* GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
+#endif /* not LOCALEDIR */
+  gsize length;
+  gint i, j, count, license = -1;
   MmaidWindow *mmaid = NULL;
-  Profile *profile;
+  GKeyFile *key_file;
+  GList *glist;
   GObject *accel, *winmenu;
   GtkAccelGroup *accel_group;
   GtkWidget *handle, *vbox;
@@ -141,75 +155,102 @@ main (int   argc,
 
   /* 初期化 */
   setlocale (LC_ALL, "");
+  gtk_set_locale ();
+  gtk_init (&argc, &argv);
 #ifdef LOCALEDIR
   bindtextdomain (PACKAGE, LOCALEDIR);
 #else /* not LOCALEDIR */
-  {
-    gchar *dirname, *path, *locale;
-
-    path = fileio_get_full_path (argv[0]);
-    dirname = g_path_get_dirname (path);
-    g_free (path);
-    locale = g_strconcat (dirname, G_DIR_SEPARATOR_S, "locale", NULL);
-    g_free (dirname);
-    bindtextdomain (PACKAGE, locale);
-    g_free (locale);
-  }
+  path = fileio_get_full_path (argv[0]);
+  dirname = g_path_get_dirname (path);
+  g_free (path);
+  locale = g_build_filename (dirname, "locale", NULL);
+  g_free (dirname);
+# ifdef G_OS_WIN32
+  if (!fileio_isfile (locale))
+    {
+      g_free (locale);
+      locale = g_win32_get_package_installation_subdirectory
+                            (NULL, argv[0], "lib" G_DIR_SEPARATOR_S "locale");
+    }
+# endif /* G_OS_WIN32 */
+# if GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
+#  if GLIB_CHECK_VERSION(2,8,0)
+  temp = g_win32_locale_filename_from_utf8 (locale);
+#  else /* not GLIB_CHECK_VERSION(2,8,0) */
+  temp = g_locale_from_utf8 (locale, -1, NULL, NULL, NULL);
+#  endif /* not GLIB_CHECK_VERSION(2,8,0) */
+  g_free (locale);
+  locale = temp;
+# endif /* GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
+  bindtextdomain (PACKAGE, locale);
+  g_free (locale);
 #endif /* not LOCALEDIR */
   bind_textdomain_codeset (PACKAGE, "UTF-8");
   textdomain (PACKAGE);
-  gtk_set_locale ();
-  gtk_init (&argc, &argv);
 
-  def_width  = gdk_screen_width ()  * 2 / 3;
+  def_width  = gdk_screen_width () * 2 / 3;
   def_height = gdk_screen_height () * 2 / 3;
   g_memset (genre, 0, sizeof (gchar *) * 256 * 256);
 
   /* ja:ファイルの履歴 */
   history = orz_history_new ();
 
-  /* ja:レジストリから読み出す */
-  profile = profile_open (NULL);
-  if (profile)
+  /* ja:キーファイルを求める */
+#ifdef G_OS_WIN32
+# if GLIB_CHECK_VERSION(2,6,0)
+  user_config = g_build_filename (g_get_user_config_dir (),
+                                    "maid.org", "mmaid", "mmaid.ini", NULL);
+# else /* not GLIB_CHECK_VERSION(2,6,0) */
+  if (SHGetSpecialFolderLocation (NULL, CSIDL_APPDATA, &pidl) == S_OK)
     {
-      gint num;
+      TCHAR szPath[MAX_PATH + 1];
 
-      if (!profile_get_value (profile, "mmaid", "license",
-                (gpointer)&license, sizeof (gboolean),
-                                                    PROFILE_VALUE_TYPE_BOOL))
-        license = FALSE;
-      if (profile_get_value (profile, "mmaid", "history",
-                (gpointer)&num, sizeof (gint), PROFILE_VALUE_TYPE_INT))
-        orz_history_set_num (ORZ_HISTORY (history), num);
-      if (!profile_get_value (profile, "mmaid", "newfile",
-                (gpointer)&newfile, sizeof (gboolean),
-                                                    PROFILE_VALUE_TYPE_BOOL))
-        newfile = FALSE;
-      if (!profile_get_value (profile, "mmaid", "second",
-                (gpointer)&second, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL))
-        second = FALSE;
-      if (!profile_get_value (profile, "mmaid", "def_hold",
-                (gpointer)&def_hold, sizeof (gboolean),
-                                                    PROFILE_VALUE_TYPE_BOOL))
-        def_hold = TRUE;
-      if (!profile_get_value (profile, "mmaid", "def_gnum",
-                (gpointer)&def_gnum, sizeof (gint), PROFILE_VALUE_TYPE_INT))
-        def_gnum = 192;
-      if (!profile_get_value (profile, "mmaid", "def_width",
-                (gpointer)&def_width, sizeof (gint), PROFILE_VALUE_TYPE_INT))
-        def_width = gdk_screen_width () * 2 / 3;
-      if (!profile_get_value (profile, "mmaid", "def_height",
-                (gpointer)&def_height, sizeof (gint), PROFILE_VALUE_TYPE_INT))
-        def_height = gdk_screen_height () * 2 / 3;
-      if (!profile_get_value (profile, "mmaid", "n_pos",
-                (gpointer)&n_pos, sizeof (gint), PROFILE_VALUE_TYPE_INT))
-        n_pos = GTK_POS_TOP;
-      if (!profile_get_value (profile, "mmaid", "playlist",
-                (gpointer)&playlist, sizeof (gint), PROFILE_VALUE_TYPE_INT))
-        playlist = 0;
-      cmd_play = profile_get_string (profile, "mmaid", "cmd_play");
-      cmd_stop = profile_get_string (profile, "mmaid", "cmd_stop");
-      charset_list = profile_get_string (profile, "mmaid", "charlist");
+      if (SHGetPathFromIDList (pidl, szPath))
+#  ifdef UNICODE
+        {
+          gchar *dir;
+
+          utf8str = g_utf16_to_utf8 (szPath, -1, NULL, NULL, NULL);
+          dir = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
+          g_free (utf8str);
+          user_config = g_build_filename (dir,
+                                    "maid.org", "mmaid", "mmaid.ini", NULL);
+          g_free (dir);
+        }
+#  else /* not UNICODE */
+        user_config = g_build_filename (szPath,
+                                    "maid.org", "mmaid", "mmaid.ini", NULL);
+#  endif /* not UNICODE */
+      CoTaskMemFree (pidl);
+    }
+  if (!user_config)
+    user_config = g_build_filename (g_get_home_dir (), "mmaid.ini", NULL);
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+#else /* not G_OS_WIN32 */
+  user_config = g_build_filename (g_get_home_dir (), ".mmaid", NULL);
+#endif /* not G_OS_WIN32 */
+
+  /* ja:レジストリから読み出す */
+  key_file = g_key_file_new ();
+  if (g_key_file_load_from_file (key_file, user_config, G_KEY_FILE_NONE, NULL))
+    {
+      license = g_key_file_get_integer (key_file, "mmaid", "license", NULL);
+      orz_history_set_num (ORZ_HISTORY (history),
+                g_key_file_get_integer (key_file, "mmaid", "history", NULL));
+      newfile = g_key_file_get_boolean (key_file, "mmaid", "newfile", NULL);
+      second = g_key_file_get_boolean (key_file, "mmaid", "second", NULL);
+      def_width = g_key_file_get_integer (key_file, "mmaid", "def_width",
+                                                                        NULL);
+      def_height = g_key_file_get_integer (key_file, "mmaid", "def_height",
+                                                                        NULL);
+      n_pos = g_key_file_get_integer (key_file, "mmaid", "n_pos", NULL);
+      def_hold = g_key_file_get_boolean (key_file, "mmaid", "def_hold", NULL);
+      def_gnum = g_key_file_get_integer (key_file, "mmaid", "def_gnum", NULL);
+      playlist = g_key_file_get_integer (key_file, "mmaid", "playlist", NULL);
+      cmd_play = g_key_file_get_string (key_file, "mmaid", "cmd_play", NULL);
+      cmd_stop = g_key_file_get_string (key_file, "mmaid", "cmd_stop", NULL);
+      charset_list = g_key_file_get_string (key_file, "mmaid", "charset_list",
+                                                                        NULL);
       if (charset_list)
         {
           gchar *tmp;
@@ -218,11 +259,11 @@ main (int   argc,
           g_free (charset_list);
           charset_list = tmp;
         }
-      utf8str = profile_get_string (profile, "mmaid", "openpath");
+      utf8str = g_key_file_get_string (key_file, "mmaid", "openpath", NULL);
       open_path = utf8str
                 ? g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL) : NULL;
       g_free (utf8str);
-      utf8str = profile_get_string (profile, "mmaid", "savepath");
+      utf8str = g_key_file_get_string (key_file, "mmaid", "savepath", NULL);
       save_path = utf8str
                 ? g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL) : NULL;
       g_free (utf8str);
@@ -230,7 +271,7 @@ main (int   argc,
       for (i = orz_history_get_num (ORZ_HISTORY (history)) - 1; i >= 0; i--)
         {
           key = g_strdup_printf ("history%d", i);
-          utf8str = profile_get_string (profile, "mmaid", key);
+          utf8str = g_key_file_get_string (key_file, "mmaid", key, NULL);
           g_free (key);
           if (utf8str)
             {
@@ -245,12 +286,13 @@ main (int   argc,
       /* ja:ジャンル */
       for (i = 0; i < 256; i++)
         {
-          section = g_strdup_printf ("genre%03d", i);
+          group_name = g_strdup_printf ("genre%03d", i);
           count = 0;
           for (j = 0; j < 256; j++)
             {
               key = g_strdup_printf ("genre%03d", j);
-              genre[i][j] = profile_get_string (profile, section, key);
+              genre[i][j] = g_key_file_get_string (key_file, group_name, key,
+                                                                        NULL);
               g_free (key);
               if (genre[i][j] && (genre[i][j])[0] == '\0')
                 {
@@ -260,23 +302,25 @@ main (int   argc,
               if (genre[i][j])
                 count++;
             }
-          g_free (section);
+          g_free (group_name);
           if (count <= 0)
             break;
           genres++;
         }
-      profile_close (profile);
     }
-  if (!license)
-    license = license_dialog ();
-  if (!license)
+  g_key_file_free (key_file);
+
+  if (license_hash (gnugpl) != license)
+    license = license_dialog (gnugpl);
+  if (license < 0)
     gtk_exit (0);
+
   /* ja:キャラクターセット */
   if (!charset_list)
     {
       const gchar *charset;
 
-      charset_list = !g_get_charset (&charset) && charset != NULL
+      charset_list = !g_get_charset (&charset) && charset
                 ? g_strdup_printf ("%s,UTF-8", charset) : g_strdup ("UTF-8");
     }
   /* ja:ジャンル */
@@ -335,7 +379,7 @@ main (int   argc,
     /* ja:ウインドウサイズ */
     array[0] = def_width;
     array[1] = def_height;
-    def[0] = gdk_screen_width ()  * 2 / 3;
+    def[0] = gdk_screen_width () * 2 / 3;
     def[1] = gdk_screen_height () * 2 / 3;
     arg_s.leng = 2;
     arg_s.array = array;
@@ -494,124 +538,100 @@ MMAID_COPYRIGHT_NOTICE"\n"
   gtk_main();
 
   /* ja:レジストリに書き込む */
-  profile = profile_open (NULL);
-  if (profile)
+  key_file = g_key_file_new ();
+  g_key_file_load_from_file (key_file, user_config, G_KEY_FILE_NONE, NULL);
+  g_key_file_set_integer (key_file, "mmaid", "license", license);
+  g_key_file_set_integer (key_file, "mmaid", "history",
+                                orz_history_get_num (ORZ_HISTORY (history)));
+  g_key_file_set_boolean (key_file, "mmaid", "newfile", newfile);
+  g_key_file_set_boolean (key_file, "mmaid", "second", second);
+  g_key_file_set_integer (key_file, "mmaid", "def_width", def_width);
+  g_key_file_set_integer (key_file, "mmaid", "def_height", def_height);
+  g_key_file_set_integer (key_file, "mmaid", "n_pos", n_pos);
+  g_key_file_set_boolean (key_file, "mmaid", "def_hold", def_hold);
+  g_key_file_set_integer (key_file, "mmaid", "def_gnum", def_gnum);
+  g_key_file_set_integer (key_file, "mmaid", "playlist", playlist);
+  g_key_file_set_string (key_file, "mmaid", "cmd_play", cmd_play);
+  g_key_file_set_string (key_file, "mmaid", "cmd_stop", cmd_stop);
+  g_key_file_set_string (key_file, "mmaid", "charset_list", charset_list);
+  if (open_path)
     {
-      gint num;
-      GList *glist;
-
-      num = orz_history_get_num (ORZ_HISTORY (history));
-      profile_set_value (profile, "mmaid", "license",
-            (gpointer)&license, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
-      profile_set_value (profile, "mmaid", "history",
-            (gpointer)&num, sizeof (gint), PROFILE_VALUE_TYPE_INT);
-      profile_set_value (profile, "mmaid", "newfile",
-            (gpointer)&newfile, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
-      profile_set_value (profile, "mmaid", "second",
-            (gpointer)&second, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
-      profile_set_value (profile, "mmaid", "def_hold",
-            (gpointer)&def_hold, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
-      profile_set_value (profile, "mmaid", "def_gnum",
-            (gpointer)&def_gnum, sizeof (gint), PROFILE_VALUE_TYPE_INT);
-      profile_set_value (profile, "mmaid", "def_width",
-            (gpointer)&def_width, sizeof (gint), PROFILE_VALUE_TYPE_INT);
-      profile_set_value (profile, "mmaid", "def_height",
-            (gpointer)&def_height, sizeof (gint), PROFILE_VALUE_TYPE_INT);
-      profile_set_value (profile, "mmaid", "n_pos",
-            (gpointer)&n_pos, sizeof (gint), PROFILE_VALUE_TYPE_INT);
-      profile_set_value (profile, "mmaid", "playlist",
-            (gpointer)&playlist, sizeof (gint), PROFILE_VALUE_TYPE_INT);
-      profile_set_value (profile, "mmaid", "cmd_play", (gpointer)cmd_play,
-                                (g_strlen (cmd_play) + 1) * sizeof (gchar),
-                                                    PROFILE_VALUE_TYPE_STRING);
-      profile_set_value (profile, "mmaid", "cmd_stop", (gpointer)cmd_stop,
-                                (g_strlen (cmd_stop) + 1) * sizeof (gchar),
-                                                    PROFILE_VALUE_TYPE_STRING);
-      profile_set_value (profile, "mmaid", "charlist", (gpointer)charset_list,
-                                (g_strlen (charset_list) + 1) * sizeof (gchar),
-                                                    PROFILE_VALUE_TYPE_STRING);
-      if (open_path)
+      utf8str = g_filename_to_utf8 (open_path, -1, NULL, NULL, NULL);
+      if (utf8str)
         {
-          utf8str = g_filename_to_utf8 (open_path, -1, NULL, NULL, NULL);
-          if (utf8str)
-            {
-              profile_set_value (profile, "mmaid", "openpath",
-                                    (gpointer)utf8str,
-                                    (g_strlen (utf8str) + 1) * sizeof (gchar),
-                                                    PROFILE_VALUE_TYPE_STRING);
-              g_free (utf8str);
-            }
+          g_key_file_set_string (key_file, "mmaid", "openpath", utf8str);
+          g_free (utf8str);
         }
-      if (save_path)
-        {
-          utf8str = g_filename_to_utf8 (save_path, -1, NULL, NULL, NULL);
-          if (utf8str)
-            {
-              profile_set_value (profile, "mmaid", "savepath",
-                                    (gpointer)utf8str,
-                                    (g_strlen (utf8str) + 1) * sizeof (gchar),
-                                                    PROFILE_VALUE_TYPE_STRING);
-              g_free (utf8str);
-            }
-        }
-      /* ja:ファイルの履歴 */
-      glist = orz_history_get_file (ORZ_HISTORY (history));
-      for (i = 0; i < g_list_length (glist); i++)
-        {
-          key = g_strdup_printf ("history%d", i);
-          utf8str = g_filename_to_utf8 (g_list_nth_data (glist, i), -1,
-                                                            NULL, NULL, NULL);
-          if (utf8str)
-            {
-              profile_set_value (profile, "mmaid", key, (gpointer)utf8str,
-                                    (g_strlen (utf8str) + 1) * sizeof (gchar),
-                                                    PROFILE_VALUE_TYPE_STRING);
-              g_free (utf8str);
-            }
-          g_free (key);
-        }
-      g_list_free (glist);
-      while (i <= ORZ_HISTORY_NUM_MAX)
-        {
-          key = g_strdup_printf ("history%d", i++);
-          profile_delete_key (profile, "mmaid", key);
-          g_free (key);
-        }
-      /* ja:ジャンル */
-      for (i = 0; i < genres; i++)
-        {
-          section = g_strdup_printf ("genre%03d", i);
-          for (j = 0; j < 256; j++)
-            {
-              key = g_strdup_printf ("genre%03d", j);
-
-              if (genre[i][j])
-                profile_set_value (profile, section, key,
-                            (gpointer)genre[i][j],
-                            (g_strlen (genre[i][j]) + 1) * sizeof (gchar),
-                                                    PROFILE_VALUE_TYPE_STRING);
-              else
-                profile_delete_key (profile, section, key);
-              g_free (key);
-            }
-          g_free (section);
-        }
-      while (i < 256)
-        {
-          section = g_strdup_printf ("genre%03d", i++);
-          profile_delete_section (profile, section);
-          g_free (section);
-        }
-      profile_close (profile);
     }
+  if (save_path)
+    {
+      utf8str = g_filename_to_utf8 (save_path, -1, NULL, NULL, NULL);
+      if (utf8str)
+        {
+          g_key_file_set_string (key_file, "mmaid", "savepath", utf8str);
+          g_free (utf8str);
+        }
+    }
+  /* ja:ファイルの履歴 */
+  glist = orz_history_get_file (ORZ_HISTORY (history));
+  for (i = 0; i < g_list_length (glist); i++)
+    {
+      key = g_strdup_printf ("history%d", i);
+      utf8str = g_filename_to_utf8 (g_list_nth_data (glist, i), -1,
+                                                            NULL, NULL, NULL);
+      if (utf8str)
+        {
+          g_key_file_set_string (key_file, "mmaid", key, utf8str);
+          g_free (utf8str);
+        }
+      g_free (key);
+    }
+  g_list_free (glist);
+  while (i <= ORZ_HISTORY_NUM_MAX)
+    {
+      key = g_strdup_printf ("history%d", i++);
+      g_key_file_remove_key (key_file, "mmaid", key, NULL);
+      g_free (key);
+    }
+  /* ja:ジャンル */
+  for (i = 0; i < genres; i++)
+    {
+      group_name = g_strdup_printf ("genre%03d", i);
+      for (j = 0; j < 256; j++)
+        {
+          key = g_strdup_printf ("genre%03d", j);
+
+          if (genre[i][j])
+            g_key_file_set_string (key_file, group_name, key, genre[i][j]);
+          else
+            g_key_file_remove_key (key_file, group_name, key, NULL);
+          g_free (key);
+        }
+      g_free (group_name);
+    }
+  while (i < 256)
+    {
+      group_name = g_strdup_printf ("genre%03d", i++);
+      g_key_file_remove_group (key_file, group_name, NULL);
+      g_free (group_name);
+    }
+  data = g_key_file_to_data (key_file, &length, NULL);
+  g_key_file_free (key_file);
+  if (data)
+    {
+      fileio_save (user_config, data, length);
+      g_free (data);
+    }
+
   for (i = 0; i < genres; i++)
     for (j = 0; j < 256; j++)
       g_free (genre[i][j]);
+  g_free (user_config);
+  g_free (clipboard_id3tag);
+  g_free (open_path);
+  g_free (save_path);
   g_free (cmd_play);
   g_free (cmd_stop);
   g_free (charset_list);
-  g_free (open_path);
-  g_free (save_path);
-  g_free (clipboard_id3tag);
   return 0;
 }
