@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include <gdk/gdkkeysyms.h>
 #include "command.h"
@@ -86,7 +86,7 @@ signal_drag_data_received (GtkWidget        *widget,
 
   i = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
   mmaid = i >= 0 ? orz_mdi_get_data (ORZ_MDI (mdi), i) : NULL;
-  files = g_strsplit (selection_data->data, "\r\n", G_MAXINT);
+  files = g_strsplit (selection_data->data, "\r\n", 0);
   for (i = 0; files[i]; i++)
     {
       file = g_filename_from_uri (files[i], NULL, NULL);
@@ -187,7 +187,7 @@ signal_selection_received (GtkWidget        *widget,
           mmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
           mm_list_get_cursor (MM_LIST (mmaid->list), &n, &c);
           mm_list_unselect_all (MM_LIST (mmaid->list));
-          tag = g_strsplit (data->data, "\n", G_MAXINT);
+          tag = g_strsplit (data->data, "\n", 0);
           for (i = 0; tag[i]; i++)
             {
               gchar **a;
@@ -195,7 +195,7 @@ signal_selection_received (GtkWidget        *widget,
               ID3Tag *id3tag;
 
               id3tag = g_malloc (sizeof (ID3Tag));
-              a = g_strsplit (tag[i], "\t", G_MAXINT);
+              a = g_strsplit (tag[i], "\t", 0);
               for (j = 0; a[j]; j++);
               if (j == 9)
                 {
@@ -260,7 +260,7 @@ signal_selection_recv (GObject   *object,
   /* ja:ウインドウサイズ */
   array[0] = def_width;
   array[1] = def_height;
-  def[0] = gdk_screen_width ()  * 2 / 3;
+  def[0] = gdk_screen_width () * 2 / 3;
   def[1] = gdk_screen_height () * 2 / 3;
   arg_s.leng = 2;
   arg_s.array = array;
